@@ -178,7 +178,7 @@ void branch_new(void){
   db_end_transaction(0);
 
   /* Do an autosync push, if requested */
-  if( !isPrivate ) autosync_loop(SYNC_PUSH, db_get_int("autosync-tries", 1));
+  if( !isPrivate ) autosync_loop(SYNC_PUSH, db_get_int("autosync-tries",1),0);
 }
 
 #if INTERFACE
@@ -388,7 +388,10 @@ static void new_brlist_page(void){
 
 /*
 ** WEBPAGE: brlist
-** Show a list of branches
+** Show a list of branches.  With no query parameters, a sortable table
+** is used to show all branches.  If query parameters are present a
+** fixed bullet list is shown.
+**
 ** Query parameters:
 **
 **     all         Show all branches
