@@ -107,10 +107,17 @@ void hyperlinked_path(
 /*
 ** WEBPAGE: dir
 **
+** Show the files and subdirectories within a single directory of the
+** source tree.  Only files for a single check-in are shown if the ci=
+** query parameter is present.  If ci= is missing, the union of files
+** across all check-ins is shown.
+**
 ** Query parameters:
 **
 **    name=PATH        Directory to display.  Optional.  Top-level if missing
 **    ci=LABEL         Show only files in this check-in.  Optional.
+**    type=TYPE        TYPE=flat: use this display
+**                     TYPE=tree: use the /tree display instead
 */
 void page_dir(void){
   char *zD = fossil_strdup(P("name"));
@@ -506,8 +513,16 @@ static void relinkTree(FileTree *pTree, FileTreeNode *pRoot){
 /*
 ** WEBPAGE: tree
 **
+** Show the files using a tree-view.  If the ci= query parameter is present
+** then show only the files for the check-in identified.  If ci= is omitted,
+** then show the union of files over all check-ins.
+**
+** The type=tree query parameter is required or else the /dir format is
+** used.
+**
 ** Query parameters:
 **
+**    type=tree        Required to prevent use of /dir format
 **    name=PATH        Directory to display.  Optional
 **    ci=LABEL         Show only files in this check-in.  Optional.
 **    re=REGEXP        Show only files matching REGEXP.  Optional.
@@ -586,7 +601,7 @@ void page_tree(void){
       linkTip = rid != symbolic_name_to_rid("tip", "ci");
       zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
       rNow = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", rid);
-      zNow = db_text("", "SELECT datetime(mtime,'localtime')"
+      zNow = db_text("", "SELECT datetime(mtime,toLocal())"
                          " FROM event WHERE objid=%d", rid);
     }else{
       zCI = 0;
@@ -594,7 +609,7 @@ void page_tree(void){
   }
   if( zCI==0 ){
     rNow = db_double(0.0, "SELECT max(mtime) FROM event");
-    zNow = db_text("", "SELECT datetime(max(mtime),'localtime') FROM event");
+    zNow = db_text("", "SELECT datetime(max(mtime),toLocal()) FROM event");
   }
 
   /* Compute the title of the page */
@@ -966,7 +981,7 @@ void test_fileage_cmd(void){
   const char *zGlob = find_option("glob",0,1);
   db_find_and_open_repository(0,0);
   verify_all_options();
-  if( g.argc!=3 ) usage("test-fileage CHECKIN");
+  if( g.argc!=3 ) usage("CHECKIN");
   mid = name_to_typed_rid(g.argv[2],"ci");
   compute_fileage(mid, zGlob);
   db_prepare(&q,
@@ -986,7 +1001,10 @@ void test_fileage_cmd(void){
 }
 
 /*
-** WEBPAGE:  fileage
+** WEBPAGE: fileage
+**
+** Show all files in a single check-in (identified by the name= query
+** parameter) in order of increasing age.
 **
 ** Parameters:
 **   name=VERSION   Selects the check-in version (default=tip).
@@ -1005,6 +1023,7 @@ void fileage_page(void){
   double baseTime;
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  if( exclude_spiders() ) return;
   zName = P("name");
   if( zName==0 ) zName = "tip";
   rid = symbolic_name_to_rid(zName, "ci");
@@ -1013,7 +1032,7 @@ void fileage_page(void){
   }
   zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", rid);
   baseTime = db_double(0.0,"SELECT mtime FROM event WHERE objid=%d", rid);
-  zNow = db_text("", "SELECT datetime(mtime,'localtime') FROM event"
+  zNow = db_text("", "SELECT datetime(mtime,toLocal()) FROM event"
                      " WHERE objid=%d", rid);
   style_submenu_element("Tree-View", "Tree-View",
                         "%R/tree?ci=%T&mtime=1&type=tree",
