@@ -75,7 +75,7 @@ void finfo_cmd(void){
       fossil_fatal("no checkout to finfo files in");
     }
     vfile_check_signature(vid, CKSIG_ENOTFILE);
-    file_tree_name(g.argv[2], &fname, 1);
+    file_tree_name(g.argv[2], &fname, 0, 1);
     db_prepare(&q,
         "SELECT pathname, deleted, rid, chnged, coalesce(origname!=pathname,0)"
         "  FROM vfile WHERE vfile.pathname=%B %s",
@@ -124,7 +124,7 @@ void finfo_cmd(void){
     /* We should be done with options.. */
     verify_all_options();
 
-    file_tree_name(g.argv[2], &fname, 1);
+    file_tree_name(g.argv[2], &fname, 0, 1);
     if( zRevision ){
       historical_version_of_file(zRevision, blob_str(&fname), &record, 0,0,0,0);
     }else{
@@ -176,7 +176,7 @@ void finfo_cmd(void){
     if( g.argc!=3 ){
       usage("?-l|--log? ?-b|--brief? FILENAME");
     }
-    file_tree_name(g.argv[2], &fname, 1);
+    file_tree_name(g.argv[2], &fname, 0, 1);
     rid = db_int(0, "SELECT rid FROM vfile WHERE pathname=%B %s",
                  &fname, filename_collation());
     if( rid==0 ){
@@ -184,7 +184,7 @@ void finfo_cmd(void){
     }
     zFilename = blob_str(&fname);
     db_prepare(&q,
-        "SELECT b.uuid, ci.uuid, date(event.mtime%s),"
+        "SELECT DISTINCT b.uuid, ci.uuid, date(event.mtime,toLocal()),"
         "       coalesce(event.ecomment, event.comment),"
         "       coalesce(event.euser, event.user),"
         "       (SELECT value FROM tagxref WHERE tagid=%d AND tagtype>0"
@@ -196,7 +196,7 @@ void finfo_cmd(void){
         "   AND event.objid=mlink.mid"
         "   AND event.objid=ci.rid"
         " ORDER BY event.mtime DESC LIMIT %d OFFSET %d",
-        timeline_utc(), TAG_BRANCH, zFilename, filename_collation(),
+        TAG_BRANCH, zFilename, filename_collation(),
         iLimit, iOffset
     );
     blob_zero(&line);
@@ -261,10 +261,10 @@ void cat_cmd(void){
   verify_all_options();
 
   for(i=2; i<g.argc; i++){
-    file_tree_name(g.argv[i], &fname, 1);
+    file_tree_name(g.argv[i], &fname, 0, 1);
     blob_zero(&content);
-    rc = historical_version_of_file(zRev, blob_str(&fname), &content, 0,0,0,0);
-    if( rc==0 ){
+    rc = historical_version_of_file(zRev, blob_str(&fname), &content, 0,0,0,2);
+    if( rc==2 ){
       fossil_fatal("no such file: %s", g.argv[i]);
     }
     blob_write_to_file(&content, "-");
@@ -290,6 +290,7 @@ void cat_cmd(void){
 **    brbg       Background color by branch name
 **    ubg        Background color by user name
 **    ci=UUID    Ancestors of a particular check-in
+**    showid     Show RID values for debugging
 */
 void finfo_page(void){
   Stmt q;
@@ -300,7 +301,6 @@ void finfo_page(void){
   int n;
   int baseCheckin;
   int fnid;
-  Bag ancestor;
   Blob title;
   Blob sql;
   HQuery url;
@@ -309,6 +309,7 @@ void finfo_page(void){
   int uBg = P("ubg")!=0;
   int fDebug = atoi(PD("debug","0"));
   int fShowId = P("showid")!=0;
+  Stmt qparent;
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
@@ -326,19 +327,17 @@ void finfo_page(void){
     style_footer();
     return;
   }
+  if( g.perm.Admin ){
+    style_submenu_element("MLink Table", "mtab", "%R/mlink?name=%t", zFilename);
+  }
   if( baseCheckin ){
-    int baseFid = db_int(0,
-      "SELECT fid FROM mlink WHERE fnid=%d AND mid=%d",
-      fnid, baseCheckin
-    );
-    bag_init(&ancestor);
-    if( baseFid ) bag_insert(&ancestor, baseFid);
+    compute_direct_ancestors(baseCheckin);
   }
   url_add_parameter(&url, "name", zFilename);
   blob_zero(&sql);
   blob_append_sql(&sql,
     "SELECT"
-    " datetime(min(event.mtime)%s),"                 /* Date of change */
+    " datetime(min(event.mtime),toLocal()),"         /* Date of change */
     " coalesce(event.ecomment, event.comment),"      /* Check-in comment */
     " coalesce(event.euser, event.user),"            /* User who made chng */
     " mlink.pid,"                                    /* Parent file rid */
@@ -354,7 +353,7 @@ void finfo_page(void){
     "  FROM mlink, event"
     " WHERE mlink.fnid=%d"
     "   AND event.objid=mlink.mid",
-    timeline_utc(), TAG_BRANCH, fnid
+    TAG_BRANCH, fnid
   );
   if( (zA = P("a"))!=0 ){
     blob_append_sql(&sql, " AND event.mtime>=julianday('%q')", zA);
@@ -364,19 +363,26 @@ void finfo_page(void){
     blob_append_sql(&sql, " AND event.mtime<=julianday('%q')", zB);
     url_add_parameter(&url, "b", zB);
   }
-  /* We only want each version of a file to appear on the graph once,
-  ** at its earliest appearance.  All the other times that it gets merged
-  ** into this or that branch can be ignored.  An exception is for when
-  ** files are deleted (when they have mlink.fid==0).  If the same file
-  ** is deleted in multiple places, we want to show each deletion, so
-  ** use a "fake fid" which is derived from the parent-fid for grouping.
-  ** The same fake-fid must be used on the graph.
-  */
-  blob_append_sql(&sql,
-    " GROUP BY"
-    "   CASE WHEN mlink.fid>0 THEN mlink.fid ELSE mlink.pid+1000000000 END"
-    " ORDER BY event.mtime DESC /*sort*/"
-  );
+  if( baseCheckin ){
+    blob_append_sql(&sql,
+      " AND mlink.mid IN (SELECT rid FROM ancestor)"
+      " GROUP BY mlink.fid"
+    );
+  }else{
+    /* We only want each version of a file to appear on the graph once,
+    ** at its earliest appearance.  All the other times that it gets merged
+    ** into this or that branch can be ignored.  An exception is for when
+    ** files are deleted (when they have mlink.fid==0).  If the same file
+    ** is deleted in multiple places, we want to show each deletion, so
+    ** use a "fake fid" which is derived from the parent-fid for grouping.
+    ** The same fake-fid must be used on the graph.
+    */
+    blob_append_sql(&sql,
+      " GROUP BY"
+      "   CASE WHEN mlink.fid>0 THEN mlink.fid ELSE mlink.pid+1000000000 END"
+    );
+  }
+  blob_append_sql(&sql, " ORDER BY event.mtime DESC /*sort*/");
   if( (n = atoi(PD("n","0")))>0 ){
     blob_append_sql(&sql, " LIMIT %d", n);
     url_add_parameter(&url, "n", P("n"));
@@ -390,8 +396,13 @@ void finfo_page(void){
   if( baseCheckin ){
     char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", baseCheckin);
     char *zLink = 	href("%R/info/%!S", zUuid);
-    blob_appendf(&title, "Ancestors of file ");
-    hyperlinked_path(zFilename, &title, zUuid, "tree", "");
+    if( n>0 ){
+      blob_appendf(&title, "First %d ancestors of file ", n);
+    }else{
+      blob_appendf(&title, "Ancestors of file ");
+    }
+    blob_appendf(&title,"<a href='%R/finfo?name=%T'>%h</a>",
+                 zFilename, zFilename);
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
     blob_appendf(&title, " from check-in %z%S</a>", zLink, zUuid);
     if( fShowId ) blob_appendf(&title, " (%d)", baseCheckin);
@@ -404,9 +415,21 @@ void finfo_page(void){
   @ <h2>%b(&title)</h2>
   blob_reset(&title);
   pGraph = graph_init();
-  @ <div id="canvas" style="position:relative;width:1px;height:1px;"
-  @  onclick="clickOnGraph(event)"></div>
   @ <table id="timelineTable" class="timelineTable">
+  if( baseCheckin ){
+    db_prepare(&qparent,
+      "SELECT DISTINCT pid FROM mlink"
+      " WHERE fid=:fid AND mid=:mid AND pid>0 AND fnid=:fnid"
+      "   AND pmid IN (SELECT rid FROM ancestor)"
+      " ORDER BY isaux /*sort*/"
+    );
+  }else{
+    db_prepare(&qparent,
+      "SELECT DISTINCT pid FROM mlink"
+      " WHERE fid=:fid AND mid=:mid AND pid>0 AND fnid=:fnid"
+      " ORDER BY isaux /*sort*/"
+    );
+  }
   while( db_step(&q)==SQLITE_ROW ){
     const char *zDate = db_column_text(&q, 0);
     const char *zCom = db_column_text(&q, 1);
@@ -424,20 +447,12 @@ void finfo_page(void){
     char zTime[10];
     int nParent = 0;
     int aParent[GR_MAX_RAIL];
-    static Stmt qparent;
 
-    if( baseCheckin && frid && !bag_find(&ancestor, frid) ) continue;
-    db_static_prepare(&qparent,
-      "SELECT DISTINCT pid FROM mlink"
-      " WHERE fid=:fid AND mid=:mid AND pid>0 AND fnid=:fnid"
-      " ORDER BY isaux /*sort*/"
-    );
     db_bind_int(&qparent, ":fid", frid);
     db_bind_int(&qparent, ":mid", fmid);
     db_bind_int(&qparent, ":fnid", fnid);
     while( db_step(&qparent)==SQLITE_ROW && nParent<ArraySize(aParent) ){
       aParent[nParent] = db_column_int(&qparent, 0);
-      if( baseCheckin ) bag_insert(&ancestor, aParent[nParent]);
       nParent++;
     }
     db_reset(&qparent);
@@ -460,7 +475,8 @@ void finfo_page(void){
     zTime[5] = 0;
     @ <tr><td class="timelineTime">
     @ %z(href("%R/timeline?c=%t",zDate))%s(zTime)</a></td>
-    @ <td class="timelineGraph"><div id="m%d(gidx)"></div></td>
+    @ <td class="timelineGraph"><div id="m%d(gidx)" class="tl-nodemark"></div>
+    @ </td>
     if( zBgClr && zBgClr[0] ){
       @ <td class="timelineTableCell" style="background-color: %h(zBgClr);">
     }else{
@@ -510,7 +526,7 @@ void finfo_page(void){
       @ %z(href("%R/blame?filename=%h&checkin=%s",z,zCkin))
       @ [blame]</a>
       @ %z(href("%R/timeline?n=200&uf=%!S",zUuid))[check-ins&nbsp;using]</a>
-      if( fpid ){
+      if( fpid>0 ){
         @ %z(href("%R/fdiff?sbs=1&v1=%!S&v2=%!S",zPUuid,zUuid))[diff]</a>
       }
     }
@@ -531,19 +547,191 @@ void finfo_page(void){
     @ </td></tr>
   }
   db_finalize(&q);
+  db_finalize(&qparent);
   if( pGraph ){
     graph_finish(pGraph, 1);
     if( pGraph->nErr ){
       graph_free(pGraph);
       pGraph = 0;
     }else{
-      int w = pGraph->mxRail*pGraph->iRailPitch + 28;
-      @ <tr><td></td><td>
-      @ <div id="grbtm" style="width:%d(w)px;"></div>
-      @     </td><td></td></tr>
+      @ <tr class="timelineBottom"><td></td><td></td><td></td></tr>
     }
   }
   @ </table>
   timeline_output_graph_javascript(pGraph, 0, 1);
   style_footer();
 }
+
+/*
+** WEBPAGE: mlink
+** URL: /mlink?name=FILENAME
+** URL: /mlink?ci=NAME
+**
+** Show all MLINK table entries for a particular file, or for
+** a particular check-in.  This screen is intended for use by developers
+** in debugging Fossil.
+*/
+void mlink_page(void){
+  const char *zFName = P("name");
+  const char *zCI = P("ci");
+  Stmt q;
+  
+  login_check_credentials();
+  if( !g.perm.Admin ){ login_needed(g.anon.Admin); return; }
+  style_header("MLINK Table");
+  if( zFName==0 && zCI==0 ){
+    @ <span class='generalError'>
+    @ Requires either a name= or ci= query parameter
+    @ </span>
+  }else if( zFName ){
+    int fnid = db_int(0,"SELECT fnid FROM filename WHERE name=%Q",zFName);
+    if( fnid<=0 ) fossil_fatal("no such file: \"%s\"", zFName);
+    db_prepare(&q,
+       "SELECT"
+       /* 0 */ "  datetime(event.mtime,toLocal()),"
+       /* 1 */ "  (SELECT uuid FROM blob WHERE rid=mlink.mid),"
+       /* 2 */ "  (SELECT uuid FROM blob WHERE rid=mlink.pmid),"
+       /* 3 */ "  isaux,"
+       /* 4 */ "  (SELECT uuid FROM blob WHERE rid=mlink.fid),"
+       /* 5 */ "  (SELECT uuid FROM blob WHERE rid=mlink.pid),"
+       /* 6 */ "  mlink.pid,"
+       /* 7 */ "  mperm,"
+       /* 8 */ "  (SELECT name FROM filename WHERE fnid=mlink.pfnid)"
+       "  FROM mlink, event"
+       " WHERE mlink.fnid=%d"
+       "   AND event.objid=mlink.mid"
+       " ORDER BY 1 DESC",
+       fnid
+    );
+    @ <h1>MLINK table for file
+    @ <a href='%R/finfo?name=%t(zFName)'>%h(zFName)</a></h1>
+    @ <div class='brlist'>
+    @ <table id='mlinktable'>
+    @ <thead><tr>
+    @ <th>Date</th>
+    @ <th>Check-in</th>
+    @ <th>Parent Check-in</th>
+    @ <th>Merge?</th>
+    @ <th>New</th>
+    @ <th>Old</th>
+    @ <th>Exe Bit?</th>
+    @ <th>Prior Name</th>
+    @ </tr></thead>
+    @ <tbody>
+    while( db_step(&q)==SQLITE_ROW ){
+      const char *zDate = db_column_text(&q,0);
+      const char *zCkin = db_column_text(&q,1);
+      const char *zParent = db_column_text(&q,2);
+      int isMerge = db_column_int(&q,3);
+      const char *zFid = db_column_text(&q,4);
+      const char *zPid = db_column_text(&q,5);
+      int isExe = db_column_int(&q,7);
+      const char *zPrior = db_column_text(&q,8);
+      @ <tr>
+      @ <td><a href='%R/timeline?c=%!S(zCkin)'>%s(zDate)</a></td>
+      @ <td><a href='%R/info/%!S(zCkin)'>%S(zCkin)</a></td>
+      if( zParent ){
+        @ <td><a href='%R/info/%!S(zPid)'>%S(zParent)</a></td>
+      }else{
+        @ <td><i>(New)</i></td>
+      }
+      @ <td align='center'>%s(isMerge?"&#x2713;":"")</td>
+      if( zFid ){
+        @ <td><a href='%R/info/%!S(zFid)'>%S(zFid)</a></td>
+      }else{
+        @ <td><i>(Deleted)</i></td>
+      }
+      if( zPid ){
+        @ <td><a href='%R/info/%!S(zPid)'>%S(zPid)</a>
+      }else if( db_column_int(&q,6)<0 ){
+        @ <td><i>(Added by merge)</i></td>
+      }else{
+        @ <td><i>(New)</i></td>
+      }
+      @ <td align='center'>%s(isExe?"&#x2713;":"")</td>
+      if( zPrior ){
+        @ <td><a href='%R/finfo?name=%t(zPrior)'>%h(zPrior)</a></td>
+      }else{
+        @ <td></td>
+      }
+      @ </tr>
+    }
+    db_finalize(&q);
+    @ </tbody>
+    @ </table>
+    @ </div>
+    output_table_sorting_javascript("mlinktable","tttxtttt",1);
+  }else{
+    int mid = name_to_rid_www("ci");
+    db_prepare(&q,
+       "SELECT"
+       /* 0 */ "  (SELECT name FROM filename WHERE fnid=mlink.fnid),"
+       /* 1 */ "  (SELECT uuid FROM blob WHERE rid=mlink.fid),"
+       /* 2 */ "  pid,"
+       /* 3 */ "  (SELECT uuid FROM blob WHERE rid=mlink.pid),"
+       /* 4 */ "  (SELECT name FROM filename WHERE fnid=mlink.pfnid),"
+       /* 5 */ "  (SELECT uuid FROM blob WHERE rid=mlink.pmid),"
+       /* 6 */ "  mperm,"
+       /* 7 */ "  isaux"
+       "  FROM mlink WHERE mid=%d ORDER BY 1",
+       mid
+    );
+    @ <h1>MLINK table for check-in %h(zCI)</h1>
+    render_checkin_context(mid, 1);
+    @ <hr>
+    @ <div class='brlist'>
+    @ <table id='mlinktable'>
+    @ <thead><tr>
+    @ <th>File</th>
+    @ <th>From</th>
+    @ <th>Merge?</th>
+    @ <th>New</th>
+    @ <th>Old</th>
+    @ <th>Exe Bit?</th>
+    @ <th>Prior Name</th>
+    @ </tr></thead>
+    @ <tbody>
+    while( db_step(&q)==SQLITE_ROW ){
+      const char *zName = db_column_text(&q,0);
+      const char *zFid = db_column_text(&q,1);
+      const char *zPid = db_column_text(&q,3);
+      const char *zPrior = db_column_text(&q,4);
+      const char *zParent = db_column_text(&q,5);
+      int isExec = db_column_int(&q,6);
+      int isAux = db_column_int(&q,7);
+      @ <tr>
+      @ <td><a href='%R/finfo?name=%t(zName)'>%h(zName)</a></td>
+      if( zParent ){
+        @ <td><a href='%R/info/%!S(zPid)'>%S(zParent)</a></td>
+      }else{
+        @ <td><i>(New)</i></td>
+      }
+      @ <td align='center'>%s(isAux?"&#x2713;":"")</td>
+      if( zFid ){
+        @ <td><a href='%R/info/%!S(zFid)'>%S(zFid)</a></td>
+      }else{
+        @ <td><i>(Deleted)</i></td>
+      }
+      if( zPid ){
+        @ <td><a href='%R/info/%!S(zPid)'>%S(zPid)</a>
+      }else if( db_column_int(&q,2)<0 ){
+        @ <td><i>(Added by merge)</i></td>
+      }else{
+        @ <td><i>(New)</i></td>
+      }
+      @ <td align='center'>%s(isExec?"&#x2713;":"")</td>
+      if( zPrior ){
+        @ <td><a href='%R/finfo?name=%t(zPrior)'>%h(zPrior)</a></td>
+      }else{
+        @ <td></td>
+      }
+      @ </tr>
+    }
+    db_finalize(&q);
+    @ </tbody>
+    @ </table>
+    @ </div>
+    output_table_sorting_javascript("mlinktable","ttxtttt",1);
+  }
+  style_footer();
+}
