@@ -104,13 +104,14 @@ struct FossilUserPerms {
 struct TclContext {
   int argc;              /* Number of original (expanded) arguments. */
   char **argv;           /* Full copy of the original (expanded) arguments. */
-  void *library;         /* The Tcl library module handle. */
+  void *hLibrary;        /* The Tcl library module handle. */
   void *xFindExecutable; /* See tcl_FindExecutableProc in th_tcl.c. */
   void *xCreateInterp;   /* See tcl_CreateInterpProc in th_tcl.c. */
   void *xDeleteInterp;   /* See tcl_DeleteInterpProc in th_tcl.c. */
   void *xFinalize;       /* See tcl_FinalizeProc in th_tcl.c. */
   Tcl_Interp *interp;    /* The on-demand created Tcl interpreter. */
   int useObjProc;        /* Non-zero if an objProc can be called directly. */
+  int useTip285;         /* Non-zero if TIP #285 is available. */
   char *setup;           /* The optional Tcl setup script. */
   void *xPreEval;        /* Optional, called before Tcl_Eval*(). */
   void *pPreContext;     /* Optional, provided to xPreEval(). */
@@ -137,6 +138,7 @@ struct Global {
   char *zLocalDbName;     /* Name of the local database */
   const char *zMainDbType;/* "configdb", "localdb", or "repository" */
   const char *zConfigDbType;  /* "configdb", "localdb", or "repository" */
+  char *zOpenRevision;    /* Check-in version to use during database open */
   int localOpen;          /* True if the local database is open */
   char *zLocalRoot;       /* The directory holding the  local database */
   int minPrefix;          /* Number of digits needed for a distinct UUID */
@@ -420,11 +422,11 @@ static void expand_args_option(int argc, void *argv){
 #if defined(_WIN32) && defined(BROKEN_MINGW_CMDLINE)
   for(i=0; i<g.argc; i++) g.argv[i] = fossil_mbcs_to_utf8(g.argv[i]);
 #else
-  for(i=0; i<g.argc; i++) g.argv[i] = fossil_filename_to_utf8(g.argv[i]);
+  for(i=0; i<g.argc; i++) g.argv[i] = fossil_path_to_utf8(g.argv[i]);
 #endif
 #if defined(_WIN32)
   GetModuleFileNameW(NULL, buf, MAX_PATH);
-  g.nameOfExe = fossil_filename_to_utf8(buf);
+  g.nameOfExe = fossil_path_to_utf8(buf);
 #else
   g.nameOfExe = g.argv[0];
 #endif
@@ -592,8 +594,8 @@ int main(int argc, char **argv)
   const char *zCmdName = "unknown";
   int idx;
   int rc;
-  if( sqlite3_libversion_number()<3008003 ){
-    fossil_fatal("Unsuitable SQLite version %s, must be at least 3.8.3",
+  if( sqlite3_libversion_number()<3010000 ){
+    fossil_fatal("Unsuitable SQLite version %s, must be at least 3.10.0",
                  sqlite3_libversion());
   }
   sqlite3_config(SQLITE_CONFIG_MULTITHREAD);
@@ -658,6 +660,7 @@ int main(int argc, char **argv)
   }else{
     const char *zChdir = find_option("chdir",0,1);
     g.isHTTP = 0;
+    g.rcvid = 0;
     g.fQuiet = find_option("quiet", 0, 0)!=0;
     g.fSqlTrace = find_option("sqltrace", 0, 0)!=0;
     g.fSqlStats = find_option("sqlstats", 0, 0)!=0;
@@ -682,15 +685,28 @@ int main(int argc, char **argv)
       fossil_fatal("unable to change directories to %s", zChdir);
     }
     if( find_option("help",0,0)!=0 ){
-      /* --help anywhere on the command line is translated into
-      ** "fossil help argv[1] argv[2]..." */
-      int i;
+      /* If --help is found anywhere on the command line, translate the command
+       * to "fossil help cmdname" where "cmdname" is the first argument that
+       * does not begin with a "-" character.  If all arguments start with "-",
+       * translate to "fossil help argv[1] argv[2]...". */
+      int i, nNewArgc;
       char **zNewArgv = fossil_malloc( sizeof(char*)*(g.argc+2) );
-      for(i=1; i<g.argc; i++) zNewArgv[i+1] = g.argv[i];
-      zNewArgv[i+1] = 0;
       zNewArgv[0] = g.argv[0];
       zNewArgv[1] = "help";
-      g.argc++;
+      for(i=1; i<g.argc; i++){
+        if( g.argv[i][0]!='-' ){
+          nNewArgc = 3;
+          zNewArgv[2] = g.argv[i];
+          zNewArgv[3] = 0;
+          break;
+        }
+      }
+      if( i==g.argc ){
+        for(i=1; i<g.argc; i++) zNewArgv[i+1] = g.argv[i];
+        nNewArgc = g.argc+1;
+        zNewArgv[i+1] = 0;
+      }
+      g.argc = nNewArgc;
       g.argv = zNewArgv;
     }
     zCmdName = g.argv[1];
@@ -863,6 +879,40 @@ const char *find_option(const char *zLong, const char *zShort, int hasArg){
   return zReturn;
 }
 
+/*
+** Look for multiple occurrences of a command-line option with the
+** corresponding argument.
+**
+** Return a malloc allocated array of pointers to the arguments.
+**
+** pnUsedArgs is used to store the number of matched arguments.
+**
+** Caller is responsible to free allocated memory.
+*/
+const char **find_repeatable_option(
+  const char *zLong,
+  const char *zShort,
+  int *pnUsedArgs
+){
+  const char *zOption;
+  const char **pzArgs = 0;
+  int nAllocArgs = 0;
+  int nUsedArgs = 0;
+
+  while( (zOption = find_option(zLong, zShort, 1))!=0 ){
+    if( pzArgs==0 && nAllocArgs==0 ){
+      nAllocArgs = 1;
+      pzArgs = fossil_malloc( nAllocArgs*sizeof(pzArgs[0]) );
+    }else if( nAllocArgs<=nUsedArgs ){
+      nAllocArgs = nAllocArgs*2;
+      pzArgs = fossil_realloc( (void *)pzArgs, nAllocArgs*sizeof(pzArgs[0]) );
+    }
+    pzArgs[nUsedArgs++] = zOption;
+  }
+  *pnUsedArgs = nUsedArgs;
+  return pzArgs;
+}
+
 /*
 ** Look for a repository command-line option.  If present, [re-]cache it in
 ** the global state and return the new pointer, freeing any previous value.
@@ -937,7 +987,7 @@ static void command_list(const char *zPrefix, int cmdMask){
 /*
 ** COMMAND: test-list-webpage
 **
-** List all web pages
+** List all web pages.
 */
 void cmd_test_webpage_list(void){
   int i, nCmd;
@@ -962,6 +1012,82 @@ const char *get_version(){
   return version;
 }
 
+/*
+** This function populates a blob with version information.  It is used by
+** the "version" command and "test-version" web page.  It assumes the blob
+** passed to it is uninitialized; otherwise, it will leak memory.
+*/
+static void get_version_blob(
+  Blob *pOut,                 /* Write the manifest here */
+  int bVerbose                /* Non-zero for full information. */
+){
+#if defined(FOSSIL_ENABLE_TCL)
+  int rc;
+  const char *zRc;
+#endif
+  blob_zero(pOut);
+  blob_appendf(pOut, "This is fossil version %s\n", get_version());
+  if( !bVerbose ) return;
+  blob_appendf(pOut, "Compiled on %s %s using %s (%d-bit)\n",
+               __DATE__, __TIME__, COMPILER_NAME, sizeof(void*)*8);
+  blob_appendf(pOut, "SQLite %s %.30s\n", sqlite3_libversion(),
+               sqlite3_sourceid());
+  blob_appendf(pOut, "Schema version %s\n", AUX_SCHEMA_MAX);
+#if defined(FOSSIL_ENABLE_MINIZ)
+  blob_appendf(pOut, "miniz %s, loaded %s\n", MZ_VERSION, mz_version());
+#else
+  blob_appendf(pOut, "zlib %s, loaded %s\n", ZLIB_VERSION, zlibVersion());
+#endif
+#if defined(FOSSIL_ENABLE_SSL)
+  blob_appendf(pOut, "SSL (%s)\n", SSLeay_version(SSLEAY_VERSION));
+#endif
+#if defined(FOSSIL_ENABLE_LEGACY_MV_RM)
+  blob_append(pOut, "LEGACY_MV_RM\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_EXEC_REL_PATHS)
+  blob_append(pOut, "EXEC_REL_PATHS\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_TH1_DOCS)
+  blob_append(pOut, "TH1_DOCS\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_TH1_HOOKS)
+  blob_append(pOut, "TH1_HOOKS\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_TCL)
+  Th_FossilInit(TH_INIT_DEFAULT | TH_INIT_FORCE_TCL);
+  rc = Th_Eval(g.interp, 0, "tclInvoke info patchlevel", -1);
+  zRc = Th_ReturnCodeName(rc, 0);
+  blob_appendf(pOut, "TCL (Tcl %s, loaded %s: %s)\n",
+    TCL_PATCH_LEVEL, zRc, Th_GetResult(g.interp, 0)
+  );
+#endif
+#if defined(USE_TCL_STUBS)
+  blob_append(pOut, "USE_TCL_STUBS\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_TCL_STUBS)
+  blob_append(pOut, "TCL_STUBS\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_TCL_PRIVATE_STUBS)
+  blob_append(pOut, "TCL_PRIVATE_STUBS\n", -1);
+#endif
+#if defined(FOSSIL_ENABLE_JSON)
+  blob_appendf(pOut, "JSON (API %s)\n", FOSSIL_JSON_API_VERSION);
+#endif
+#if defined(BROKEN_MINGW_CMDLINE)
+  blob_append(pOut, "MBCS_COMMAND_LINE\n", -1);
+#else
+  blob_append(pOut, "UNICODE_COMMAND_LINE\n", -1);
+#endif
+#if defined(FOSSIL_DYNAMIC_BUILD)
+  blob_append(pOut, "DYNAMIC_BUILD\n", -1);
+#else
+  blob_append(pOut, "STATIC_BUILD\n", -1);
+#endif
+#if defined(USE_SEE)
+  blob_append(pOut, "USE_SEE\n", -1);
+#endif
+}
+
 /*
 ** This function returns the user-agent string for Fossil, for
 ** use in HTTP(S) requests.
@@ -972,6 +1098,7 @@ const char *get_user_agent(){
   return version;
 }
 
+
 /*
 ** COMMAND: version
 **
@@ -983,65 +1110,38 @@ const char *get_user_agent(){
 ** with
 */
 void version_cmd(void){
-  int verboseFlag = 0;
-
-  fossil_print("This is fossil version %s\n", get_version());
-  verboseFlag = find_option("verbose","v",0)!=0;
+  Blob versionInfo;
+  int verboseFlag = find_option("verbose","v",0)!=0;
 
   /* We should be done with options.. */
   verify_all_options();
+  get_version_blob(&versionInfo, verboseFlag);
+  fossil_print("%s", blob_str(&versionInfo));
+}
 
-  if(!verboseFlag){
-    return;
-  }else{
-#if defined(FOSSIL_ENABLE_TCL)
-    int rc;
-    const char *zRc;
-#endif
-    fossil_print("Compiled on %s %s using %s (%d-bit)\n",
-                 __DATE__, __TIME__, COMPILER_NAME, sizeof(void*)*8);
-    fossil_print("SQLite %s %.30s\n", sqlite3_libversion(), sqlite3_sourceid());
-    fossil_print("Schema version %s\n", AUX_SCHEMA_MAX);
-#if defined(FOSSIL_ENABLE_MINIZ)
-    fossil_print("miniz %s, loaded %s\n", MZ_VERSION, mz_version());
-#else
-    fossil_print("zlib %s, loaded %s\n", ZLIB_VERSION, zlibVersion());
-#endif
-#if defined(FOSSIL_ENABLE_SSL)
-    fossil_print("SSL (%s)\n", SSLeay_version(SSLEAY_VERSION));
-#endif
-#if defined(FOSSIL_ENABLE_TH1_DOCS)
-    fossil_print("TH1_DOCS\n");
-#endif
-#if defined(FOSSIL_ENABLE_TH1_HOOKS)
-    fossil_print("TH1_HOOKS\n");
-#endif
-#if defined(FOSSIL_ENABLE_TCL)
-    Th_FossilInit(TH_INIT_DEFAULT | TH_INIT_FORCE_TCL);
-    rc = Th_Eval(g.interp, 0, "tclInvoke info patchlevel", -1);
-    zRc = Th_ReturnCodeName(rc, 0);
-    fossil_print("TCL (Tcl %s, loaded %s: %s)\n",
-      TCL_PATCH_LEVEL, zRc, Th_GetResult(g.interp, 0)
-    );
-#endif
-#if defined(USE_TCL_STUBS)
-    fossil_print("USE_TCL_STUBS\n");
-#endif
-#if defined(FOSSIL_ENABLE_TCL_STUBS)
-    fossil_print("TCL_STUBS\n");
-#endif
-#if defined(FOSSIL_ENABLE_TCL_PRIVATE_STUBS)
-    fossil_print("TCL_PRIVATE_STUBS\n");
-#endif
-#if defined(FOSSIL_ENABLE_JSON)
-    fossil_print("JSON (API %s)\n", FOSSIL_JSON_API_VERSION);
-#endif
-#if defined(BROKEN_MINGW_CMDLINE)
-    fossil_print("MBCS_COMMAND_LINE\n");
-#else
-    fossil_print("UNICODE_COMMAND_LINE\n");
-#endif
-  }
+
+/*
+** WEBPAGE: test-version
+**
+** Show the version information for Fossil.
+**
+** Query parameters:
+**
+**    verbose       Show all available details.
+*/
+void test_version_page(void){
+  Blob versionInfo;
+  int verboseFlag;
+
+  login_check_credentials();
+  if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  verboseFlag = P("verbose")!=0;
+  style_header("Version Information");
+  get_version_blob(&versionInfo, verboseFlag);
+  @ <blockquote><pre>
+  @ %h(blob_str(&versionInfo))
+  @ </pre></blockquote>
+  style_footer();
 }
 
 
@@ -1052,7 +1152,7 @@ void version_cmd(void){
 **    or: %fossil COMMAND --help
 **
 ** Display information on how to use COMMAND.  To display a list of
-** available commands one of:
+** available commands use one of:
 **
 **    %fossil help              Show common commands
 **    %fossil help -a|--all     Show both common and auxiliary commands
@@ -1128,9 +1228,54 @@ void help_cmd(void){
   putchar('\n');
 }
 
+/*
+** COMMAND: test-all-help
+** 
+** Usage: %fossil test-all-help ?OPTIONS?
+**
+** Show help text for commands and pages.  Useful for proof-reading.
+** Defaults to just the CLI commands.  Specify --www to see only the
+** web pages, or --everything to see both commands and pages.
+**
+** Options:
+**    -e|--everything   Show all commands and pages.
+**    -t|--test         Include test- commands
+**    -w|--www          Show WWW pages.
+*/
+void test_all_help_cmd(void){
+  int i;
+  int mask = CMDFLAG_1ST_TIER | CMDFLAG_2ND_TIER;
+
+  if( find_option("www","w",0) ){
+    mask = CMDFLAG_WEBPAGE;
+  }
+  if( find_option("everything","e",0) ){
+    mask = CMDFLAG_1ST_TIER | CMDFLAG_2ND_TIER | CMDFLAG_WEBPAGE;
+  }
+  if( find_option("test","t",0) ){
+    mask |= CMDFLAG_TEST;
+  }
+  fossil_print("Help text for:\n");
+  if( mask & CMDFLAG_1ST_TIER ) fossil_print(" * Commands\n");
+  if( mask & CMDFLAG_2ND_TIER ) fossil_print(" * Auxiliary commands\n");
+  if( mask & CMDFLAG_TEST )     fossil_print(" * Test commands\n");
+  if( mask & CMDFLAG_WEBPAGE )  fossil_print(" * Web pages\n");
+  fossil_print("---\n");
+  for(i=0; i<count(aCommand); i++){
+    if( (aCommand[i].cmdFlags & mask)==0 ) continue;
+    fossil_print("# %s\n", aCommand[i].zName);
+    fossil_print("%s\n\n", aCmdHelp[i].zText);
+  }
+  fossil_print("---\n");
+  version_cmd();
+}
+
 /*
 ** WEBPAGE: help
-** URL: /help/CMD
+** URL: /help?name=CMD
+**
+** Show the built-in help text for CMD.  CMD can be a command-line interface
+** command or a page name from the web interface.
 */
 void help_page(void){
   const char *zCmd = P("cmd");
@@ -1140,9 +1285,14 @@ void help_page(void){
   if( zCmd ){
     int rc, idx;
     char *z, *s, *d;
-    const char *zCmdOrPage = ('/'==*zCmd) ? "page" : "command";
     style_submenu_element("Command-List", "Command-List", "%s/help", g.zTop);
-    @ <h1>The "%s(zCmd)" %s(zCmdOrPage):</h1>
+    if( *zCmd=='/' ){
+      /* Some of the webpages require query parameters in order to work.
+      ** @ <h1>The "<a href='%R%s(zCmd)'>%s(zCmd)</a>" page:</h1> */
+      @ <h1>The "%s(zCmd)" page:</h1>
+    }else{
+      @ <h1>The "%s(zCmd)" command:</h1>
+    }
     rc = name_search(zCmd, aCommand, count(aCommand), 0, &idx);
     if( rc==1 ){
       @ unknown command: %s(zCmd)
@@ -1198,7 +1348,6 @@ void help_page(void){
     @ </tr></table>
 
     @ <h1>Available web UI pages:</h1>
-    @ (Only pages with help text are linked.)
     @ <table border="0"><tr>
     for(i=j=0; i<count(aCommand); i++){
       const char *z = aCommand[i].zName;
@@ -1300,7 +1449,13 @@ static void set_base_url(const char *zAltBase){
   if( zAltBase ){
     int i, n, c;
     g.zTop = g.zBaseURL = mprintf("%s", zAltBase);
-    if( memcmp(g.zTop, "http://", 7)!=0 && memcmp(g.zTop,"https://",8)!=0 ){
+    if( strncmp(g.zTop, "http://", 7)==0 ){
+      /* it is HTTP, replace prefix with HTTPS. */
+      g.zHttpsURL = mprintf("https://%s", &g.zTop[7]);
+    }else if( strncmp(g.zTop, "https://", 8)==0 ){
+      /* it is already HTTPS, use it. */
+      g.zHttpsURL = mprintf("%s", g.zTop);
+    }else{
       fossil_fatal("argument to --baseurl should be 'http://host/path'"
                    " or 'https://host/path'");
     }
@@ -1366,7 +1521,8 @@ NORETURN void fossil_redirect_home(void){
 ** is a directory, of that directory.
 **
 ** The noJail flag means that the chroot jail is not entered.  But
-** privileges are still lowered to that of the the user-id and group-id.
+** privileges are still lowered to that of the user-id and group-id
+** of the repository file.
 */
 static char *enter_chroot_jail(char *zRepo, int noJail){
 #if !defined(_WIN32)
@@ -1433,10 +1589,16 @@ static int repo_list_page(void){
   db_multi_exec("CREATE TABLE sfile(x TEXT);");
   db_multi_exec("CREATE TABLE vfile(pathname);");
   vfile_scan(&base, blob_size(&base), 0, 0, 0);
-  db_multi_exec("DELETE FROM sfile WHERE x NOT GLOB '*.fossil'");
+  db_multi_exec("DELETE FROM sfile WHERE x NOT GLOB '*[^/].fossil'");
   n = db_int(0, "SELECT count(*) FROM sfile");
   if( n>0 ){
     Stmt q;
+    @ <html>
+    @ <head>
+    @ <base href="%s(g.zBaseURL)/" />
+    @ <title>Repository List</title>
+    @ </head>
+    @ <body>
     @ <h1>Available Repositories:</h1>
     @ <ol>
     db_prepare(&q, "SELECT x, substr(x,-7,-100000)||'/home'"
@@ -1444,9 +1606,11 @@ static int repo_list_page(void){
     while( db_step(&q)==SQLITE_ROW ){
       const char *zName = db_column_text(&q, 0);
       const char *zUrl = db_column_text(&q, 1);
-      @ <li><a href="%h(zUrl)">%h(zName)</a></li>
+      @ <li><a href="%R/%h(zUrl)" target="_blank">%h(zName)</a></li>
     }
     @ </ol>
+    @ </body>
+    @ </html>
     cgi_reply();
   }
   sqlite3_close(g.db);
@@ -1466,7 +1630,7 @@ static int repo_list_page(void){
 ** Process the webpage specified by the PATH_INFO or REQUEST_URI
 ** environment variable.
 **
-** If the repository is not known, the a search is done through the
+** If the repository is not known, then a search is done through the
 ** file hierarchy rooted at g.zRepositoryName for a suitable repository
 ** with a name of $prefix.fossil, where $prefix is any prefix of PATH_INFO.
 ** Or, if an ordinary file named $prefix is found, and $prefix matches
@@ -1482,14 +1646,32 @@ static void process_one_web_page(
   int allowRepoList           /* Send repo list for "/" URL */
 ){
   const char *zPathInfo;
+  const char *zDirPathInfo;
   char *zPath = NULL;
   int idx;
   int i;
 
+  /* Handle universal query parameters */
+  if( PB("utc") ){
+    g.fTimeFormat = 1;
+  }else if( PB("localtime") ){
+    g.fTimeFormat = 2;
+  }
+
   /* If the repository has not been opened already, then find the
   ** repository based on the first element of PATH_INFO and open it.
   */
-  zPathInfo = PD("PATH_INFO","");
+  zDirPathInfo = zPathInfo = PD("PATH_INFO","");
+  /* For the PATH_INFO that will be used to help build the final
+  ** g.zBaseURL and g.zTop (only), skip over the initial directory
+  ** portion of PATH_INFO; otherwise, it may be duplicated.
+  */
+  if( g.zTop ){
+    int nTop = strlen(g.zTop);
+    if ( strncmp(zDirPathInfo, g.zTop, nTop)==0 ){
+      zDirPathInfo += nTop;
+    }
+  }
   if( !g.repositoryOpen ){
     char *zRepo, *zToFree;
     const char *zOldScript = PD("SCRIPT_NAME", "");
@@ -1521,9 +1703,17 @@ static void process_one_web_page(
         szFile = 1;
         break;
       }
-      if( szFile==0 ){
+      if( szFile==0 && sqlite3_strglob("*/.fossil",zRepo)!=0 ){
         if( zRepo[0]=='/' && zRepo[1]=='/' ){ zRepo++; j--; }
         szFile = file_size(zRepo);
+        /* this should only be set from the --baseurl option, not CGI  */
+        if( g.zBaseURL && g.zBaseURL[0]!=0 && g.zTop && g.zTop[0]!=0 &&
+            file_isdir(g.zRepositoryName)==1 ){
+          if( zPathInfo==zDirPathInfo ){
+            g.zBaseURL = mprintf("%s%.*s", g.zBaseURL, i, zPathInfo);
+            g.zTop = mprintf("%s%.*s", g.zTop, i, zPathInfo);
+          }
+        }
       }
       if( szFile<0 && i>0 ){
         const char *zMimetype;
@@ -1553,12 +1743,12 @@ static void process_one_web_page(
 
       if( szFile<1024 ){
         set_base_url(0);
-        if( zNotFound ){
-          cgi_redirect(zNotFound);
-        }else if( strcmp(zPathInfo,"/")==0
+        if( strcmp(zPathInfo,"/")==0
                   && allowRepoList
                   && repo_list_page() ){
           /* Will return a list of repositories */
+        }else if( zNotFound ){
+          cgi_redirect(zNotFound);
         }else{
 #ifdef FOSSIL_ENABLE_JSON
           if(g.json.isJsonMode){
@@ -1953,12 +2143,14 @@ void cmd_cgi(void){
       blob_reset(&value);
       continue;
     }
-    if( blob_eq(&key, "setenv:") && blob_token(&line, &value)
-            && blob_token(&line, &value2) ){
+    if( blob_eq(&key, "setenv:") && blob_token(&line, &value) ){
       /* setenv: NAME VALUE
+      ** setenv: NAME
       **
-      ** Sets environment variable NAME to VALUE
+      ** Sets environment variable NAME to VALUE.  If VALUE is omitted, then
+      ** the environment variable is unset.
       */
+      blob_token(&line,&value2);
       fossil_setenv(blob_str(&value), blob_str(&value2));
       blob_reset(&value);
       blob_reset(&value2);
@@ -2024,8 +2216,8 @@ void cmd_cgi(void){
 ** that will be used by a server, or else it is a directory that
 ** contains multiple repositories that can be served.  If g.argv[arg]
 ** is a directory, the repositories it contains must be named
-** "*.fossil".  If g.argv[arg] does not exists, then we must be within
-** a check-out and the repository to be served is the repository of
+** "*.fossil".  If g.argv[arg] does not exist, then we must be within
+** an open check-out and the repository serve is the repository of
 ** that check-out.
 **
 ** Open the repository to be served if it is known.  If g.argv[arg] is
@@ -2033,15 +2225,39 @@ void cmd_cgi(void){
 ** the name of that directory and the specific repository will be
 ** opened later by process_one_web_page() based on the content of
 ** the PATH_INFO variable.
+**
+** If the fCreate flag is set, then create the repository if it
+** does not already exist.
 */
-static void find_server_repository(int arg){
+static void find_server_repository(int arg, int fCreate){
   if( g.argc<=arg ){
     db_must_be_within_tree();
-  }else if( file_isdir(g.argv[arg])==1 ){
-    g.zRepositoryName = mprintf("%s", g.argv[arg]);
-    file_simplify_name(g.zRepositoryName, -1, 0);
   }else{
-    db_open_repository(g.argv[arg]);
+    const char *zRepo = g.argv[arg];
+    int isDir = file_isdir(zRepo);
+    if( isDir==1 ){
+      g.zRepositoryName = mprintf("%s", zRepo);
+      file_simplify_name(g.zRepositoryName, -1, 0);
+    }else{
+      if( isDir==0 && fCreate ){
+        const char *zPassword;
+        db_create_repository(zRepo);
+        db_open_repository(zRepo);
+        db_begin_transaction();
+        db_initial_setup(0, "now", g.zLogin);
+        db_end_transaction(0);
+        fossil_print("project-id: %s\n", db_get("project-code", 0));
+        fossil_print("server-id:  %s\n", db_get("server-code", 0));
+        zPassword = db_text(0, "SELECT pw FROM user WHERE login=%Q", g.zLogin);
+        fossil_print("admin-user: %s (initial password is \"%s\")\n",
+                     g.zLogin, zPassword);
+        cache_initialize();
+        g.zLogin = 0;
+        g.userUid = 0;
+      }else{
+        db_open_repository(zRepo);
+      }
+    }
   }
 }
 
@@ -2050,7 +2266,7 @@ static void find_server_repository(int arg){
 **
 **        fossil http INFILE OUTFILE IPADDR ?REPOSITORY?
 **
-** The argv==6 form is used by the win32 server only.
+** The argv==6 form (with no options) is used by the win32 server only.
 **
 ** COMMAND: http*
 **
@@ -2062,8 +2278,8 @@ static void find_server_repository(int arg){
 ** repository.
 **
 ** If REPOSITORY is a directory that contains one or more repositories,
-** either directly in REPOSITORY itself, or in subdirectories, and
-** with names of the form "*.fossil" then the a prefix of the URL pathname
+** either directly in REPOSITORY itself or in subdirectories, and
+** with names of the form "*.fossil" then a prefix of the URL pathname
 ** selects from among the various repositories.  If the pathname does
 ** not select a valid repository and the --notfound option is available,
 ** then the server redirects (HTTP code 302) to the URL of --notfound.
@@ -2135,7 +2351,6 @@ void cmd_http(void){
   }
   zHost = find_option("host", 0, 1);
   if( zHost ) cgi_replace_parameter("HTTP_HOST",zHost);
-  g.cgiOutput = 1;
 
   /* We should be done with options.. */
   verify_all_options();
@@ -2143,16 +2358,17 @@ void cmd_http(void){
   if( g.argc!=2 && g.argc!=3 && g.argc!=5 && g.argc!=6 ){
     fossil_fatal("no repository specified");
   }
+  g.cgiOutput = 1;
   g.fullHttpReply = 1;
   if( g.argc>=5 ){
     g.httpIn = fossil_fopen(g.argv[2], "rb");
     g.httpOut = fossil_fopen(g.argv[3], "wb");
     zIpAddr = g.argv[4];
-    find_server_repository(5);
+    find_server_repository(5, 0);
   }else{
     g.httpIn = stdin;
     g.httpOut = stdout;
-    find_server_repository(2);
+    find_server_repository(2, 0);
   }
   if( zIpAddr==0 ){
     zIpAddr = cgi_ssh_remote_addr(0);
@@ -2188,8 +2404,12 @@ void ssh_request_loop(const char *zIpAddr, Glob *FileGlob){
 ** Note that the following command is used by ssh:// processing.
 **
 ** COMMAND: test-http
+**
 ** Works like the http command but gives setup permission to all users.
 **
+** Options:
+**   --th-trace          trace TH1 execution (for debugging purposes)
+**
 */
 void cmd_test_http(void){
   const char *zIpAddr;    /* IP address of remote client */
@@ -2199,7 +2419,7 @@ void cmd_test_http(void){
   g.useLocalauth = 1;
   g.httpIn = stdin;
   g.httpOut = stdout;
-  find_server_repository(2);
+  find_server_repository(2, 0);
   g.cgiOutput = 1;
   g.fullHttpReply = 1;
   zIpAddr = cgi_ssh_remote_addr(0);
@@ -2243,7 +2463,7 @@ static int binaryOnPath(const char *zBinary){
 ** COMMAND: ui
 **
 ** Usage: %fossil server ?OPTIONS? ?REPOSITORY?
-**    Or: %fossil ui ?OPTIONS? ?REPOSITORY?
+**    or: %fossil ui ?OPTIONS? ?REPOSITORY?
 **
 ** Open a socket and begin listening and responding to HTTP requests on
 ** TCP port 8080, or on any other TCP port defined by the -P or
@@ -2256,8 +2476,8 @@ static int binaryOnPath(const char *zBinary){
 ** only process HTTP traffic from the local machine.
 **
 ** The REPOSITORY can be a directory (aka folder) that contains one or
-** more repositories with names ending in ".fossil".  In this case, the
-** a prefix of the URL pathname is used to search the directory for an
+** more repositories with names ending in ".fossil".  In this case, a
+** prefix of the URL pathname is used to search the directory for an
 ** appropriate repository.  To thwart mischief, the pathname in the URL must
 ** contain only alphanumerics, "_", "/", "-", and ".", and no "-" may
 ** occur after "/", and every "." must be surrounded on both sides by
@@ -2278,10 +2498,14 @@ static int binaryOnPath(const char *zBinary){
 **
 ** Options:
 **   --baseurl URL       Use URL as the base (useful for reverse proxies)
+**   --create            Create a new REPOSITORY if it does not already exist
+**   --page PAGE         Start "ui" on PAGE.  ex: --page "timeline?y=ci"
 **   --files GLOBLIST    Comma-separated list of glob patterns for static files
 **   --localauth         enable automatic login for requests from localhost
 **   --localhost         listen on 127.0.0.1 only (always true for "ui")
+**   --https             signal a request coming in via https
 **   --nojail            Drop root privileges but do not enter the chroot jail
+**   --nossl             signal that no SSL connections are available
 **   --notfound URL      Redirect
 **   -P|--port TCPPORT   listen to request on port TCPPORT
 **   --th-trace          trace TH1 execution (for debugging purposes)
@@ -2303,10 +2527,12 @@ void cmd_webserver(void){
 #if !defined(_WIN32)
   int noJail;               /* Do not enter the chroot jail */
 #endif
-  int allowRepoList;        /* List repositories on URL "/" */
-  const char *zAltBase;     /* Argument to the --baseurl option */
-  const char *zFileGlob;    /* Static content must match this */
-  char *zIpAddr = 0;        /* Bind to this IP address */
+  int allowRepoList;         /* List repositories on URL "/" */
+  const char *zAltBase;      /* Argument to the --baseurl option */
+  const char *zFileGlob;     /* Static content must match this */
+  char *zIpAddr = 0;         /* Bind to this IP address */
+  int fCreate = 0;           /* The --create flag */
+  const char *zInitPage = 0; /* Start on this page.  --page option */
 
 #if defined(_WIN32)
   const char *zStopperFile;    /* Name of file used to terminate server */
@@ -2328,13 +2554,25 @@ void cmd_webserver(void){
   g.useLocalauth = find_option("localauth", 0, 0)!=0;
   Th_InitTraceLog();
   zPort = find_option("port", "P", 1);
+  isUiCmd = g.argv[1][0]=='u';
+  if( isUiCmd ){
+    zInitPage = find_option("page", 0, 1);
+  }
   zNotFound = find_option("notfound", 0, 1);
   allowRepoList = find_option("repolist",0,0)!=0;
   zAltBase = find_option("baseurl", 0, 1);
+  fCreate = find_option("create",0,0)!=0;
   if( find_option("scgi", 0, 0)!=0 ) flags |= HTTP_SERVER_SCGI;
   if( zAltBase ){
     set_base_url(zAltBase);
   }
+  g.sslNotAvailable = find_option("nossl", 0, 0)!=0;
+  if( find_option("https",0,0)!=0 ){
+    cgi_replace_parameter("HTTPS","on");
+  }else{
+    /* without --https, defaults to not available. */
+    g.sslNotAvailable = 1;
+  }
   if( find_option("localhost", 0, 0)!=0 ){
     flags |= HTTP_SERVER_LOCALHOST;
   }
@@ -2343,13 +2581,19 @@ void cmd_webserver(void){
   verify_all_options();
 
   if( g.argc!=2 && g.argc!=3 ) usage("?REPOSITORY?");
-  isUiCmd = g.argv[1][0]=='u';
   if( isUiCmd ){
     flags |= HTTP_SERVER_LOCALHOST|HTTP_SERVER_REPOLIST;
     g.useLocalauth = 1;
     allowRepoList = 1;
   }
-  find_server_repository(2);
+  find_server_repository(2, fCreate);
+  if( zInitPage==0 ){
+    if( isUiCmd && g.localOpen ){
+      zInitPage = "timeline?c=current";
+    }else{
+      zInitPage = "";
+    }
+  }
   if( zPort ){
     int i;
     for(i=strlen(zPort)-1; i>=0 && zPort[i]!=':'; i--){}
@@ -2383,27 +2627,26 @@ void cmd_webserver(void){
     zBrowser = db_get("web-browser", "open");
 #endif
     if( zIpAddr ){
-      zBrowserCmd = mprintf("%s http://%s:%%d/ &", zBrowser, zIpAddr);
+      zBrowserCmd = mprintf("%s http://%s:%%d/%s &",
+                            zBrowser, zIpAddr, zInitPage);
     }else{
-      zBrowserCmd = mprintf("%s http://localhost:%%d/ &", zBrowser);
+      zBrowserCmd = mprintf("%s http://localhost:%%d/%s &",
+                            zBrowser, zInitPage);
     }
-    if( g.repositoryOpen ) flags |= HTTP_SERVER_HAD_REPOSITORY;
-    if( g.localOpen ) flags |= HTTP_SERVER_HAD_CHECKOUT;
-  }else{
-    db_setup_server_and_project_codes(1);
   }
+  if( g.repositoryOpen ) flags |= HTTP_SERVER_HAD_REPOSITORY;
+  if( g.localOpen ) flags |= HTTP_SERVER_HAD_CHECKOUT;
   db_close(1);
   if( cgi_http_server(iPort, mxPort, zBrowserCmd, zIpAddr, flags) ){
     fossil_fatal("unable to listen on TCP socket %d", iPort);
   }
-  g.sslNotAvailable = 1;
   g.httpIn = stdin;
   g.httpOut = stdout;
   if( g.fHttpTrace || g.fSqlTrace ){
     fprintf(stderr, "====== SERVER pid %d =======\n", getpid());
   }
   g.cgiOutput = 1;
-  find_server_repository(2);
+  find_server_repository(2, 0);
   g.zRepositoryName = enter_chroot_jail(g.zRepositoryName, noJail);
   if( flags & HTTP_SERVER_SCGI ){
     cgi_handle_scgi_request();
@@ -2416,26 +2659,28 @@ void cmd_webserver(void){
   if( isUiCmd ){
     zBrowser = db_get("web-browser", "start");
     if( zIpAddr ){
-      zBrowserCmd = mprintf("%s http://%s:%%d/ &", zBrowser, zIpAddr);
+      zBrowserCmd = mprintf("%s http://%s:%%d/%s &",
+                            zBrowser, zIpAddr, zInitPage);
     }else{
-      zBrowserCmd = mprintf("%s http://localhost:%%d/ &", zBrowser);
+      zBrowserCmd = mprintf("%s http://localhost:%%d/%s &",
+                            zBrowser, zInitPage);
     }
-    if( g.repositoryOpen ) flags |= HTTP_SERVER_HAD_REPOSITORY;
-    if( g.localOpen ) flags |= HTTP_SERVER_HAD_CHECKOUT;
   }
+  if( g.repositoryOpen ) flags |= HTTP_SERVER_HAD_REPOSITORY;
+  if( g.localOpen ) flags |= HTTP_SERVER_HAD_CHECKOUT;
   db_close(1);
   if( allowRepoList ){
     flags |= HTTP_SERVER_REPOLIST;
   }
-  if( win32_http_service(iPort, zNotFound, zFileGlob, flags) ){
-    win32_http_server(iPort, mxPort, zBrowserCmd,
-                      zStopperFile, zNotFound, zFileGlob, zIpAddr, flags);
+  if( win32_http_service(iPort, zAltBase, zNotFound, zFileGlob, flags) ){
+    win32_http_server(iPort, mxPort, zBrowserCmd, zStopperFile,
+                      zAltBase, zNotFound, zFileGlob, zIpAddr, flags);
   }
 #endif
 }
 
 /*
-** COMMAND:  test-echo
+** COMMAND: test-echo
 **
 ** Usage:  %fossil test-echo [--hex] ARGS...
 **
@@ -2443,7 +2688,7 @@ void cmd_webserver(void){
 ** wildcard expansion behavior of the host shell can be investigated.
 **
 ** With the --hex option, show the output as hexadecimal.  This can be used
-** to verify the fossil_filename_to_utf8() routine on Windows and Mac.
+** to verify the fossil_path_to_utf8() routine on Windows and Mac.
 */
 void test_echo_cmd(void){
   int i, j;
