@@ -92,6 +92,7 @@ set src {
   moderate
   name
   path
+  piechart
   pivot
   popen
   pqueue
@@ -155,12 +156,16 @@ set SQLITE_OPTIONS {
   -DNDEBUG=1
   -DSQLITE_OMIT_LOAD_EXTENSION=1
   -DSQLITE_ENABLE_LOCKING_STYLE=0
+  -DSQLITE_LIKE_DOESNT_MATCH_BLOBS=1
   -DSQLITE_THREADSAFE=0
   -DSQLITE_DEFAULT_FILE_FORMAT=4
   -DSQLITE_OMIT_DEPRECATED
   -DSQLITE_ENABLE_EXPLAIN_COMMENTS
   -DSQLITE_ENABLE_FTS4
   -DSQLITE_ENABLE_FTS3_PARENTHESIS
+  -DSQLITE_ENABLE_DBSTAT_VTAB
+  -DSQLITE_ENABLE_JSON1
+  -DSQLITE_ENABLE_FTS5
 }
 #lappend SQLITE_OPTIONS -DSQLITE_ENABLE_FTS3=1
 #lappend SQLITE_OPTIONS -DSQLITE_ENABLE_STAT4
@@ -171,6 +176,7 @@ set SQLITE_OPTIONS {
 #
 set SHELL_OPTIONS {
   -Dmain=sqlite3_shell
+  -DSQLITE_SHELL_IS_UTF8=1
   -DSQLITE_OMIT_LOAD_EXTENSION=1
   -DUSE_SYSTEM_SQLITE=$(USE_SYSTEM_SQLITE)
   -DSQLITE_SHELL_DBNAME_PROC=fossil_open
@@ -301,11 +307,21 @@ $(OBJDIR)/mkversion:	$(SRCDIR)/mkversion.c
 $(OBJDIR)/codecheck1:	$(SRCDIR)/codecheck1.c
 	$(BCC) -o $(OBJDIR)/codecheck1 $(SRCDIR)/codecheck1.c
 
-# WARNING. DANGER. Running the test suite modifies the repository the
-# build is done from, i.e. the checkout belongs to. Do not sync/push
-# the repository after running the tests.
+# Run the test suite.
+# Other flags that can be included in TESTFLAGS are:
+#
+#  -halt     Stop testing after the first failed test
+#  -keep     Keep the temporary workspace for debugging
+#  -prot     Write a detailed log of the tests to the file ./prot
+#  -verbose  Include even more details in the output
+#  -quiet    Hide most output from the terminal
+#  -strict   Treat known bugs as failures
+#
+# TESTFLAGS can also include names of specific test files to limit
+# the run to just those test cases.
+#
 test:	$(OBJDIR) $(APPNAME)
-	$(TCLSH) $(SRCDIR)/../test/tester.tcl $(APPNAME)
+	$(TCLSH) $(SRCDIR)/../test/tester.tcl $(APPNAME) -quiet $(TESTFLAGS)
 
 $(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(SRCDIR)/../VERSION $(OBJDIR)/mkversion
 	$(OBJDIR)/mkversion $(SRCDIR)/../manifest.uuid \
@@ -325,8 +341,8 @@ MINIZ_OPTIONS = <<<MINIZ_OPTIONS>>>
 # to 1. If it is set to 1, then there is no need to build or link
 # the sqlite3.o object. Instead, the system SQLite will be linked
 # using -lsqlite3.
-SQLITE3_OBJ.1 =
 SQLITE3_OBJ.0 = $(OBJDIR)/sqlite3.o
+SQLITE3_OBJ.1 =
 SQLITE3_OBJ.  = $(SQLITE3_OBJ.0)
 
 # The FOSSIL_ENABLE_MINIZ variable may be undefined, set to 0, or
@@ -335,12 +351,40 @@ SQLITE3_OBJ.  = $(SQLITE3_OBJ.0)
 MINIZ_OBJ.0 =
 MINIZ_OBJ.1 = $(OBJDIR)/miniz.o
 MINIZ_OBJ.  = $(MINIZ_OBJ.0)
+
+# The USE_LINENOISE variable may be undefined, set to 0, or set
+# to 1. If it is set to 0, then there is no need to build or link
+# the linenoise.o object.
+LINENOISE_DEF.0 =
+LINENOISE_DEF.1 = -DHAVE_LINENOISE
+LINENOISE_DEF.  = $(LINENOISE_DEF.0)
+LINENOISE_OBJ.0 =
+LINENOISE_OBJ.1 = $(OBJDIR)/linenoise.o
+LINENOISE_OBJ.  = $(LINENOISE_OBJ.0)
+
+# The USE_SEE variable may be undefined, 0 or 1.  If undefined or
+# 0, ordinary SQLite is used.  If 1, then sqlite3-see.c (not part of
+# the source tree) is used and extra flags are provided to enable
+# the SQLite Encryption Extension.
+SQLITE3_SRC.0 = sqlite3.c
+SQLITE3_SRC.1 = sqlite3-see.c
+SQLITE3_SRC. = sqlite3.c
+SQLITE3_SRC = $(SRCDIR)/$(SQLITE3_SRC.$(USE_SEE))
+SQLITE3_SHELL_SRC.0 = shell.c
+SQLITE3_SHELL_SRC.1 = shell-see.c
+SQLITE3_SHELL_SRC. = shell.c
+SQLITE3_SHELL_SRC = $(SRCDIR)/$(SQLITE3_SHELL_SRC.$(USE_SEE))
+SEE_FLAGS.0 =
+SEE_FLAGS.1 = -DSQLITE_HAS_CODEC
+SEE_FLAGS. =
+SEE_FLAGS = $(SEE_FLAGS.$(USE_SEE))
 }]
 
 writeln [string map [list <<<NEXT_LINE>>> \\] {
 EXTRAOBJ = <<<NEXT_LINE>>>
  $(SQLITE3_OBJ.$(USE_SYSTEM_SQLITE)) <<<NEXT_LINE>>>
  $(MINIZ_OBJ.$(FOSSIL_ENABLE_MINIZ)) <<<NEXT_LINE>>>
+ $(LINENOISE_OBJ.$(USE_LINENOISE)) <<<NEXT_LINE>>>
  $(OBJDIR)/shell.o <<<NEXT_LINE>>>
  $(OBJDIR)/th.o <<<NEXT_LINE>>>
  $(OBJDIR)/th_lang.o <<<NEXT_LINE>>>
@@ -397,11 +441,15 @@ foreach s [lsort $src] {
   writeln "\$(OBJDIR)/$s.h:\t\$(OBJDIR)/headers\n"
 }
 
-writeln "\$(OBJDIR)/sqlite3.o:\t\$(SRCDIR)/sqlite3.c"
-writeln "\t\$(XTCC) \$(SQLITE_OPTIONS) \$(SQLITE_CFLAGS) -c \$(SRCDIR)/sqlite3.c -o \$@\n"
+writeln "\$(OBJDIR)/sqlite3.o:\t\$(SQLITE3_SRC)"
+writeln "\t\$(XTCC) \$(SQLITE_OPTIONS) \$(SQLITE_CFLAGS) \$(SEE_FLAGS) \\"
+writeln "\t\t-c \$(SQLITE3_SRC) -o \$@"
 
-writeln "\$(OBJDIR)/shell.o:\t\$(SRCDIR)/shell.c \$(SRCDIR)/sqlite3.h"
-writeln "\t\$(XTCC) \$(SHELL_OPTIONS) \$(SHELL_CFLAGS) -c \$(SRCDIR)/shell.c -o \$@\n"
+writeln "\$(OBJDIR)/shell.o:\t\$(SQLITE3_SHELL_SRC) \$(SRCDIR)/sqlite3.h"
+writeln "\t\$(XTCC) \$(SHELL_OPTIONS) \$(SHELL_CFLAGS) \$(LINENOISE_DEF.\$(USE_LINENOISE)) -c \$(SQLITE3_SHELL_SRC) -o \$@\n"
+
+writeln "\$(OBJDIR)/linenoise.o:\t\$(SRCDIR)/linenoise.c \$(SRCDIR)/linenoise.h"
+writeln "\t\$(XTCC) -c \$(SRCDIR)/linenoise.c -o \$@\n"
 
 writeln "\$(OBJDIR)/th.o:\t\$(SRCDIR)/th.c"
 writeln "\t\$(XTCC) -c \$(SRCDIR)/th.c -o \$@\n"
@@ -452,6 +500,10 @@ writeln {#!/usr/bin/make
 # This is a makefile for use on Cygwin/Darwin/FreeBSD/Linux/Windows using
 # MinGW or MinGW-w64.
 #
+# Some of the special options which can be passed to make
+#   USE_WINDOWS=1    if building under a windows command prompt
+#   X64=1            if using an unprefixed 64-bit mingw compiler
+#
 
 #### Select one of MinGW, MinGW-w64 (32-bit) or MinGW-w64 (64-bit) compilers.
 #    By default, this is an empty string (i.e. use the native compiler).
@@ -472,12 +524,20 @@ SRCDIR = src
 #
 OBJDIR = wbld
 
+#### C compiler for use in building executables that will run on
+#    the platform that is doing the build.  This is used to compile
+#    code-generator programs as part of the build process.  See TCC
+#    and TCCEXE below for the C compiler for building the finished
+#    binary.
+#
+BCCEXE = gcc
+
 #### C Compiler and options for use in building executables that
 #    will run on the platform that is doing the build.  This is used
 #    to compile code-generator programs as part of the build process.
 #    See TCC below for the C compiler for building the finished binary.
 #
-BCC = gcc
+BCC = $(BCCEXE)
 
 #### Enable compiling with debug symbols (much larger binary)
 #
@@ -496,6 +556,14 @@ BCC = gcc
 #
 # FOSSIL_BUILD_SSL = 1
 
+#### Enable relative paths in external diff/gdiff
+#
+# FOSSIL_ENABLE_EXEC_REL_PATHS = 1
+
+#### Enable legacy treatment of mv/rm (skip checkout files)
+#
+# FOSSIL_ENABLE_LEGACY_MV_RM = 1
+
 #### Enable TH1 scripts in embedded documentation files
 #
 # FOSSIL_ENABLE_TH1_DOCS = 1
@@ -520,6 +588,10 @@ BCC = gcc
 #
 # USE_SYSTEM_SQLITE = 1
 
+#### Use the SQLite Encryption Extension
+#
+# USE_SEE = 1
+
 #### Use the miniz compression library
 #
 # FOSSIL_ENABLE_MINIZ = 1
@@ -561,15 +633,15 @@ ifndef X64
 SSLCONFIG = mingw
 ifndef FOSSIL_ENABLE_MINIZ
 ZLIBCONFIG = LOC="-DASMV -DASMINF" OBJA="inffas86.o match.o"
-LIBTARGETS = $(ZLIBDIR)/inffas86.o $(ZLIBDIR)/match.o
+ZLIBTARGETS = $(ZLIBDIR)/inffas86.o $(ZLIBDIR)/match.o
 else
 ZLIBCONFIG =
-LIBTARGETS =
+ZLIBTARGETS =
 endif
 else
 SSLCONFIG = mingw64
 ZLIBCONFIG =
-LIBTARGETS =
+ZLIBTARGETS =
 endif
 
 #### Disable creation of the OpenSSL shared libraries.  Also, disable support
@@ -589,7 +661,7 @@ endif
 #    to create a hard link between an "openssl-1.x" sub-directory of the
 #    Fossil source code directory and the target OpenSSL source directory.
 #
-OPENSSLDIR = $(SRCDIR)/../compat/openssl-1.0.2
+OPENSSLDIR = $(SRCDIR)/../compat/openssl-1.0.2h
 OPENSSLINCDIR = $(OPENSSLDIR)/include
 OPENSSLLIBDIR = $(OPENSSLDIR)
 
@@ -630,25 +702,35 @@ LIBTCL = -ltcl86
 TCLTARGET = binaries
 endif
 
-#### C Compile and options for use in building executables that
-#    will run on the target platform.  This is usually the same
+#### C compiler for use in building executables that will run on the
+#    target platform.  This is usually the same as BCCEXE, unless you
+#    are cross-compiling.  This C compiler builds the finished binary
+#    for fossil.  See BCC and BCCEXE above for the C compiler for
+#    building intermediate code-generator tools.
+#
+TCCEXE = gcc
+
+#### C compiler and options for use in building executables that will
+#    run on the target platform.  This is usually the almost the same
 #    as BCC, unless you are cross-compiling.  This C compiler builds
 #    the finished binary for fossil.  The BCC compiler above is used
 #    for building intermediate code-generator tools.
 #
-TCC = $(PREFIX)gcc -Os -Wall
-
-#### When not using the miniz compression library, zlib is required.
-#
-ifndef FOSSIL_ENABLE_MINIZ
-TCC += -L$(ZLIBDIR) -I$(ZINCDIR)
-endif
+TCC = $(PREFIX)$(TCCEXE) -Wall
 
 #### Add the necessary command line options to build with debugging
 #    symbols, if enabled.
 #
 ifdef FOSSIL_ENABLE_SYMBOLS
 TCC += -g
+else
+TCC += -Os
+endif
+
+#### When not using the miniz compression library, zlib is required.
+#
+ifndef FOSSIL_ENABLE_MINIZ
+TCC += -L$(ZLIBDIR) -I$(ZINCDIR)
 endif
 
 #### Compile resources for use in building executables that will run
@@ -695,6 +777,18 @@ TCC += -DFOSSIL_ENABLE_SSL=1
 RCC += -DFOSSIL_ENABLE_SSL=1
 endif
 
+# With relative paths in external diff/gdiff
+ifdef FOSSIL_ENABLE_EXEC_REL_PATHS
+TCC += -DFOSSIL_ENABLE_EXEC_REL_PATHS=1
+RCC += -DFOSSIL_ENABLE_EXEC_REL_PATHS=1
+endif
+
+# With legacy treatment of mv/rm
+ifdef FOSSIL_ENABLE_LEGACY_MV_RM
+TCC += -DFOSSIL_ENABLE_LEGACY_MV_RM=1
+RCC += -DFOSSIL_ENABLE_LEGACY_MV_RM=1
+endif
+
 # With TH1 embedded docs support
 ifdef FOSSIL_ENABLE_TH1_DOCS
 TCC += -DFOSSIL_ENABLE_TH1_DOCS=1
@@ -731,10 +825,20 @@ TCC += -DFOSSIL_ENABLE_JSON=1
 RCC += -DFOSSIL_ENABLE_JSON=1
 endif
 
-#### We add the -static option here so that we can build a static
-#    executable that will run in a chroot jail.
+# With SQLite Encryption Extension support
+ifdef USE_SEE
+TCC += -DUSE_SEE=1
+RCC += -DUSE_SEE=1
+endif
+
+#### The option -static has no effect on MinGW(-w64), only dynamic
+#    executables can be built when linking with MSVCRT.  OpenSSL
+#    (optional) and zlib (required) however are always linked in
+#    statically.  Therefore, the FOSSIL_DYNAMIC_BUILD option does
+#    not really apply to MinGW (i.e. since ALL external libraries
+#    are NOT linked dynamically).
 #
-LIB = -static
+# LIB = -static
 
 #### MinGW: If available, use the Unicode capable runtime startup code.
 #
@@ -931,8 +1035,8 @@ $(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(MKVERSIO
 # to 1. If it is set to 1, then there is no need to build or link
 # the sqlite3.o object. Instead, the system SQLite will be linked
 # using -lsqlite3.
-SQLITE3_OBJ.1 =
 SQLITE3_OBJ.0 = $(OBJDIR)/sqlite3.o
+SQLITE3_OBJ.1 =
 SQLITE3_OBJ.  = $(SQLITE3_OBJ.0)
 
 # The FOSSIL_ENABLE_MINIZ variable may be undefined, set to 0, or
@@ -941,6 +1045,23 @@ SQLITE3_OBJ.  = $(SQLITE3_OBJ.0)
 MINIZ_OBJ.0 =
 MINIZ_OBJ.1 = $(OBJDIR)/miniz.o
 MINIZ_OBJ.  = $(MINIZ_OBJ.0)
+
+# The USE_SEE variable may be undefined, 0 or 1.  If undefined or
+# 0, ordinary SQLite is used.  If 1, then sqlite3-see.c (not part of
+# the source tree) is used and extra flags are provided to enable
+# the SQLite Encryption Extension.
+SQLITE3_SRC.0 = sqlite3.c
+SQLITE3_SRC.1 = sqlite3-see.c
+SQLITE3_SRC. = sqlite3.c
+SQLITE3_SRC = $(SRCDIR)/$(SQLITE3_SRC.$(USE_SEE))
+SQLITE3_SHELL_SRC.0 = shell.c
+SQLITE3_SHELL_SRC.1 = shell-see.c
+SQLITE3_SHELL_SRC. = shell.c
+SQLITE3_SHELL_SRC = $(SRCDIR)/$(SQLITE3_SHELL_SRC.$(USE_SEE))
+SEE_FLAGS.0 =
+SEE_FLAGS.1 = -DSQLITE_HAS_CODEC
+SEE_FLAGS. =
+SEE_FLAGS = $(SEE_FLAGS.$(USE_SEE))
 }
 
 writeln [string map [list <<<NEXT_LINE>>> \\] {
@@ -955,38 +1076,39 @@ EXTRAOBJ = <<<NEXT_LINE>>>
 }]
 
 writeln {
-zlib:
-	$(MAKE) -C $(ZLIBDIR) PREFIX=$(PREFIX) $(ZLIBCONFIG) -f win32/Makefile.gcc libz.a
-
-clean-zlib:
-	$(MAKE) -C $(ZLIBDIR) PREFIX=$(PREFIX) -f win32/Makefile.gcc clean
-
 $(ZLIBDIR)/inffas86.o:
 	$(TCC) -c -o $@ -DASMINF -I$(ZLIBDIR) -O3 $(ZLIBDIR)/contrib/inflate86/inffas86.c
 
 $(ZLIBDIR)/match.o:
 	$(TCC) -c -o $@ -DASMV $(ZLIBDIR)/contrib/asm686/match.S
 
+zlib:	$(ZLIBTARGETS)
+	$(MAKE) -C $(ZLIBDIR) PREFIX=$(PREFIX) CC=$(PREFIX)$(TCCEXE) $(ZLIBCONFIG) -f win32/Makefile.gcc libz.a
 
-ifndef FOSSIL_ENABLE_MINIZ
-LIBTARGETS += zlib
+clean-zlib:
+	$(MAKE) -C $(ZLIBDIR) PREFIX=$(PREFIX) CC=$(PREFIX)$(TCCEXE) -f win32/Makefile.gcc clean
+
+ifdef FOSSIL_ENABLE_MINIZ
+BLDTARGETS =
+else
+BLDTARGETS = zlib
 endif
 
-openssl:	$(LIBTARGETS)
+openssl:	$(BLDTARGETS)
 	cd $(OPENSSLLIBDIR);./Configure --cross-compile-prefix=$(PREFIX) $(SSLCONFIG)
-	$(MAKE) -C $(OPENSSLLIBDIR) build_libs
+	$(MAKE) -C $(OPENSSLLIBDIR) CC=$(TCCEXE) build_libs
 
 clean-openssl:
-	$(MAKE) -C $(OPENSSLLIBDIR) clean
+	$(MAKE) -C $(OPENSSLLIBDIR) CC=$(TCCEXE) clean
 
 tcl:
 	cd $(TCLSRCDIR)/win;./configure
-	$(MAKE) -C $(TCLSRCDIR)/win $(TCLTARGET)
+	$(MAKE) -C $(TCLSRCDIR)/win CC=$(TCCEXE) $(TCLTARGET)
 
 clean-tcl:
-	$(MAKE) -C $(TCLSRCDIR)/win distclean
+	$(MAKE) -C $(TCLSRCDIR)/win CC=$(TCCEXE) distclean
 
-APPTARGETS += $(LIBTARGETS)
+APPTARGETS += $(BLDTARGETS)
 
 ifdef FOSSIL_BUILD_SSL
 APPTARGETS += openssl
@@ -994,7 +1116,7 @@ endif
 
 $(APPNAME):	$(APPTARGETS) $(OBJDIR)/headers $(CODECHECK1) $(OBJ) $(EXTRAOBJ) $(OBJDIR)/fossil.o
 	$(CODECHECK1) $(TRANS_SRC)
-	$(TCC) -o $@ $(OBJ) $(EXTRAOBJ) $(LIB) $(OBJDIR)/fossil.o
+	$(TCC) -o $@ $(OBJ) $(EXTRAOBJ) $(OBJDIR)/fossil.o $(LIB)
 
 # This rule prevents make from using its default rules to try build
 # an executable named "manifest" out of the file named "manifest.c"
@@ -1049,11 +1171,14 @@ foreach s [lsort $src] {
   writeln "\$(OBJDIR)/${s}.h:\t\$(OBJDIR)/headers\n"
 }
 
+writeln {MINGW_OPTIONS = -D_HAVE__MINGW_H
+}
+
 set SQLITE_WIN32_OPTIONS $SQLITE_OPTIONS
 lappend SQLITE_WIN32_OPTIONS -DSQLITE_WIN32_NO_ANSI
 
 set MINGW_SQLITE_OPTIONS $SQLITE_WIN32_OPTIONS
-lappend MINGW_SQLITE_OPTIONS -D_HAVE__MINGW_H
+lappend MINGW_SQLITE_OPTIONS {$(MINGW_OPTIONS)}
 lappend MINGW_SQLITE_OPTIONS -DSQLITE_USE_MALLOC_H
 lappend MINGW_SQLITE_OPTIONS -DSQLITE_USE_MSIZE
 
@@ -1066,15 +1191,16 @@ writeln "SHELL_OPTIONS = [join $SHELL_WIN32_OPTIONS $j]\n"
 set j " \\\n                "
 writeln "MINIZ_OPTIONS = [join $MINIZ_WIN32_OPTIONS $j]\n"
 
-writeln "\$(OBJDIR)/sqlite3.o:\t\$(SRCDIR)/sqlite3.c \$(SRCDIR)/../win/Makefile.mingw"
-writeln "\t\$(XTCC) \$(SQLITE_OPTIONS) \$(SQLITE_CFLAGS) -c \$(SRCDIR)/sqlite3.c -o \$@\n"
+writeln "\$(OBJDIR)/sqlite3.o:\t\$(SQLITE3_SRC) \$(SRCDIR)/../win/Makefile.mingw"
+writeln "\t\$(XTCC) \$(SQLITE_OPTIONS) \$(SQLITE_CFLAGS) \$(SEE_FLAGS) \\"
+writeln "\t\t-c \$(SQLITE3_SRC) -o \$@\n"
 
 writeln "\$(OBJDIR)/cson_amalgamation.o:\t\$(SRCDIR)/cson_amalgamation.c"
 writeln "\t\$(XTCC) -c \$(SRCDIR)/cson_amalgamation.c -o \$@\n"
 writeln "\$(OBJDIR)/json.o \$(OBJDIR)/json_artifact.o \$(OBJDIR)/json_branch.o \$(OBJDIR)/json_config.o \$(OBJDIR)/json_diff.o \$(OBJDIR)/json_dir.o \$(OBJDIR)/jsos_finfo.o \$(OBJDIR)/json_login.o \$(OBJDIR)/json_query.o \$(OBJDIR)/json_report.o \$(OBJDIR)/json_status.o \$(OBJDIR)/json_tag.o \$(OBJDIR)/json_timeline.o \$(OBJDIR)/json_user.o \$(OBJDIR)/json_wiki.o : \$(SRCDIR)/json_detail.h\n"
 
-writeln "\$(OBJDIR)/shell.o:\t\$(SRCDIR)/shell.c \$(SRCDIR)/sqlite3.h \$(SRCDIR)/../win/Makefile.mingw"
-writeln "\t\$(XTCC) \$(SHELL_OPTIONS) \$(SHELL_CFLAGS) -c \$(SRCDIR)/shell.c -o \$@\n"
+writeln "\$(OBJDIR)/shell.o:\t\$(SQLITE3_SHELL_SRC) \$(SRCDIR)/sqlite3.h \$(SRCDIR)/../win/Makefile.mingw"
+writeln "\t\$(XTCC) \$(SHELL_OPTIONS) \$(SHELL_CFLAGS) -c \$(SQLITE3_SHELL_SRC) -o \$@\n"
 
 writeln "\$(OBJDIR)/th.o:\t\$(SRCDIR)/th.c"
 writeln "\t\$(XTCC) -c \$(SRCDIR)/th.c -o \$@\n"
@@ -1294,67 +1420,150 @@ P       = .pdb
 PERLDIR = C:\Perl\bin
 PERL    = perl.exe
 
-# Uncomment to enable debug symbols
-# DEBUG = 1
+# Enable debugging symbols?
+!ifndef DEBUG
+DEBUG = 0
+!endif
 
-# Uncomment to support Windows XP with Visual Studio 201x
-# FOSSIL_ENABLE_WINXP = 1
+# Build the OpenSSL libraries?
+!ifndef FOSSIL_BUILD_SSL
+FOSSIL_BUILD_SSL = 0
+!endif
 
-# Uncomment to enable JSON API
-# FOSSIL_ENABLE_JSON = 1
+# Build the included zlib library?
+!ifndef FOSSIL_BUILD_ZLIB
+FOSSIL_BUILD_ZLIB = 1
+!endif
 
-# Uncomment to enable miniz usage
-# FOSSIL_ENABLE_MINIZ = 1
+# Link everything except SQLite dynamically?
+!ifndef FOSSIL_DYNAMIC_BUILD
+FOSSIL_DYNAMIC_BUILD = 0
+!endif
 
-# Uncomment to enable SSL support
-# FOSSIL_ENABLE_SSL = 1
+# Enable relative paths in external diff/gdiff?
+!ifndef FOSSIL_ENABLE_EXEC_REL_PATHS
+FOSSIL_ENABLE_EXEC_REL_PATHS = 0
+!endif
 
-# Uncomment to build SSL libraries
-# FOSSIL_BUILD_SSL = 1
+# Enable the JSON API?
+!ifndef FOSSIL_ENABLE_JSON
+FOSSIL_ENABLE_JSON = 0
+!endif
 
-# Uncomment to enable TH1 scripts in embedded documentation files
-# FOSSIL_ENABLE_TH1_DOCS = 1
+# Enable legacy treatment of the mv/rm commands?
+!ifndef FOSSIL_ENABLE_LEGACY_MV_RM
+FOSSIL_ENABLE_LEGACY_MV_RM = 0
+!endif
 
-# Uncomment to enable TH1 hooks
-# FOSSIL_ENABLE_TH1_HOOKS = 1
+# Enable use of miniz instead of zlib?
+!ifndef FOSSIL_ENABLE_MINIZ
+FOSSIL_ENABLE_MINIZ = 0
+!endif
 
-# Uncomment to enable Tcl support
-# FOSSIL_ENABLE_TCL = 1
+# Enable OpenSSL support?
+!ifndef FOSSIL_ENABLE_SSL
+FOSSIL_ENABLE_SSL = 0
+!endif
 
-!ifdef FOSSIL_ENABLE_SSL
-SSLDIR    = $(B)\compat\openssl-1.0.2
+# Enable the Tcl integration subsystem?
+!ifndef FOSSIL_ENABLE_TCL
+FOSSIL_ENABLE_TCL = 0
+!endif
+
+# Enable TH1 scripts in embedded documentation files?
+!ifndef FOSSIL_ENABLE_TH1_DOCS
+FOSSIL_ENABLE_TH1_DOCS = 0
+!endif
+
+# Enable TH1 hooks for commands and web pages?
+!ifndef FOSSIL_ENABLE_TH1_HOOKS
+FOSSIL_ENABLE_TH1_HOOKS = 0
+!endif
+
+# Enable support for Windows XP with Visual Studio 201x?
+!ifndef FOSSIL_ENABLE_WINXP
+FOSSIL_ENABLE_WINXP = 0
+!endif
+
+# Enable support for the SQLite Encryption Extension?
+!ifndef USE_SEE
+USE_SEE = 0
+!endif
+
+!if $(FOSSIL_ENABLE_SSL)!=0
+SSLDIR    = $(B)\compat\openssl-1.0.2h
 SSLINCDIR = $(SSLDIR)\inc32
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLLIBDIR = $(SSLDIR)\out32dll
+!else
 SSLLIBDIR = $(SSLDIR)\out32
+!endif
 SSLLFLAGS = /nologo /opt:ref /debug
 SSLLIB    = ssleay32.lib libeay32.lib user32.lib gdi32.lib
 !if "$(PLATFORM)"=="amd64" || "$(PLATFORM)"=="x64"
 !message Using 'x64' platform for OpenSSL...
-# BUGBUG (OpenSSL): Apparently, using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN64A no-asm no-ssl2 no-ssl3 no-shared
-SSLCONFIG = VC-WIN64A no-asm no-shared
+# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
+# SSLCONFIG = VC-WIN64A no-asm no-ssl2 no-ssl3
+SSLCONFIG = VC-WIN64A no-asm
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLCONFIG = $(SSLCONFIG) shared
+!else
+SSLCONFIG = $(SSLCONFIG) no-shared
+!endif
 SSLSETUP  = ms\do_win64a.bat
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLNMAKE  = ms\ntdll.mak all
+!else
 SSLNMAKE  = ms\nt.mak all
+!endif
+# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
+!if $(FOSSIL_DYNAMIC_BUILD)==0
 SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3
+!endif
 !elseif "$(PLATFORM)"=="ia64"
 !message Using 'ia64' platform for OpenSSL...
-# BUGBUG (OpenSSL): Apparently, using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN64I no-asm no-ssl2 no-ssl3 no-shared
-SSLCONFIG = VC-WIN64I no-asm no-shared
+# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
+# SSLCONFIG = VC-WIN64I no-asm no-ssl2 no-ssl3
+SSLCONFIG = VC-WIN64I no-asm
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLCONFIG = $(SSLCONFIG) shared
+!else
+SSLCONFIG = $(SSLCONFIG) no-shared
+!endif
 SSLSETUP  = ms\do_win64i.bat
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLNMAKE  = ms\ntdll.mak all
+!else
 SSLNMAKE  = ms\nt.mak all
+!endif
+# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
+!if $(FOSSIL_DYNAMIC_BUILD)==0
 SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3
+!endif
 !else
 !message Assuming 'x86' platform for OpenSSL...
-# BUGBUG (OpenSSL): Apparently, using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN32 no-asm no-ssl2 no-ssl3 no-shared
-SSLCONFIG = VC-WIN32 no-asm no-shared
+# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
+# SSLCONFIG = VC-WIN32 no-asm no-ssl2 no-ssl3
+SSLCONFIG = VC-WIN32 no-asm
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLCONFIG = $(SSLCONFIG) shared
+!else
+SSLCONFIG = $(SSLCONFIG) no-shared
+!endif
 SSLSETUP  = ms\do_ms.bat
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+SSLNMAKE  = ms\ntdll.mak all
+!else
 SSLNMAKE  = ms\nt.mak all
+!endif
+# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
+!if $(FOSSIL_DYNAMIC_BUILD)==0
 SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3
 !endif
 !endif
+!endif
 
-!ifdef FOSSIL_ENABLE_TCL
+!if $(FOSSIL_ENABLE_TCL)!=0
 TCLDIR    = $(B)\compat\tcl-8.6
 TCLSRCDIR = $(TCLDIR)
 TCLINCDIR = $(TCLSRCDIR)\generic
@@ -1363,26 +1572,37 @@ TCLINCDIR = $(TCLSRCDIR)\generic
 # zlib options
 ZINCDIR   = $(B)\compat\zlib
 ZLIBDIR   = $(B)\compat\zlib
+
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+ZLIB      = zdll.lib
+!else
 ZLIB      = zlib.lib
+!endif
 
 INCL      = /I. /I$(SRCDIR) /I$B\win\include
 
-!ifndef FOSSIL_ENABLE_MINIZ
+!if $(FOSSIL_ENABLE_MINIZ)==0
 INCL      = $(INCL) /I$(ZINCDIR)
 !endif
 
-!ifdef FOSSIL_ENABLE_SSL
+!if $(FOSSIL_ENABLE_SSL)!=0
 INCL      = $(INCL) /I$(SSLINCDIR)
 !endif
 
-!ifdef FOSSIL_ENABLE_TCL
+!if $(FOSSIL_ENABLE_TCL)!=0
 INCL      = $(INCL) /I$(TCLINCDIR)
 !endif
 
 CFLAGS    = /nologo
-LDFLAGS   = /NODEFAULTLIB:msvcrt /MANIFEST:NO
+LDFLAGS   =
 
-!ifdef FOSSIL_ENABLE_WINXP
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+LDFLAGS   = $(LDFLAGS) /MANIFEST
+!else
+LDFLAGS   = $(LDFLAGS) /NODEFAULTLIB:msvcrt /MANIFEST:NO
+!endif
+
+!if $(FOSSIL_ENABLE_WINXP)!=0
 XPCFLAGS  = $(XPCFLAGS) /D_USING_V110_SDK71_=1
 CFLAGS    = $(CFLAGS) $(XPCFLAGS)
 !if "$(PLATFORM)"=="amd64" || "$(PLATFORM)"=="x64"
@@ -1393,52 +1613,82 @@ XPLDFLAGS = $(XPLDFLAGS) /SUBSYSTEM:CONSOLE,5.01
 LDFLAGS   = $(LDFLAGS) $(XPLDFLAGS)
 !endif
 
-!ifdef DEBUG
-CFLAGS    = $(CFLAGS) /Zi /MTd /Od
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+!if $(DEBUG)!=0
+CRTFLAGS = /MDd
+!else
+CRTFLAGS = /MD
+!endif
+!else
+!if $(DEBUG)!=0
+CRTFLAGS = /MTd
+!else
+CRTFLAGS = /MT
+!endif
+!endif
+
+!if $(DEBUG)!=0
+CFLAGS    = $(CFLAGS) /Zi $(CRTFLAGS) /Od
 LDFLAGS   = $(LDFLAGS) /DEBUG
 !else
-CFLAGS    = $(CFLAGS) /MT /O2
+CFLAGS    = $(CFLAGS) $(CRTFLAGS) /O2
 !endif
 
 BCC       = $(CC) $(CFLAGS)
 TCC       = $(CC) /c $(CFLAGS) $(MSCDEF) $(INCL)
-RCC       = rc /D_WIN32 /D_MSC_VER $(MSCDEF) $(INCL)
+RCC       = $(RC) /D_WIN32 /D_MSC_VER $(MSCDEF) $(INCL)
+MTC       = mt
 LIBS      = ws2_32.lib advapi32.lib
 LIBDIR    =
 
-!ifndef FOSSIL_ENABLE_MINIZ
+!if $(FOSSIL_DYNAMIC_BUILD)!=0
+TCC       = $(TCC) /DFOSSIL_DYNAMIC_BUILD=1
+RCC       = $(RCC) /DFOSSIL_DYNAMIC_BUILD=1
+!endif
+
+!if $(FOSSIL_ENABLE_MINIZ)==0
 LIBS      = $(LIBS) $(ZLIB)
 LIBDIR    = $(LIBDIR) /LIBPATH:$(ZLIBDIR)
 !endif
 
-!ifdef FOSSIL_ENABLE_MINIZ
+!if $(FOSSIL_ENABLE_MINIZ)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_MINIZ=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_MINIZ=1
 !endif
 
-!ifdef FOSSIL_ENABLE_JSON
+!if $(FOSSIL_ENABLE_JSON)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_JSON=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_JSON=1
 !endif
 
-!ifdef FOSSIL_ENABLE_SSL
+!if $(FOSSIL_ENABLE_SSL)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_SSL=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_SSL=1
 LIBS      = $(LIBS) $(SSLLIB)
 LIBDIR    = $(LIBDIR) /LIBPATH:$(SSLLIBDIR)
 !endif
 
-!ifdef FOSSIL_ENABLE_TH1_DOCS
+!if $(FOSSIL_ENABLE_EXEC_REL_PATHS)!=0
+TCC       = $(TCC) /DFOSSIL_ENABLE_EXEC_REL_PATHS=1
+RCC       = $(RCC) /DFOSSIL_ENABLE_EXEC_REL_PATHS=1
+!endif
+
+!if $(FOSSIL_ENABLE_LEGACY_MV_RM)!=0
+TCC       = $(TCC) /DFOSSIL_ENABLE_LEGACY_MV_RM=1
+RCC       = $(RCC) /DFOSSIL_ENABLE_LEGACY_MV_RM=1
+!endif
+
+!if $(FOSSIL_ENABLE_TH1_DOCS)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_TH1_DOCS=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_TH1_DOCS=1
 !endif
 
-!ifdef FOSSIL_ENABLE_TH1_HOOKS
+!if $(FOSSIL_ENABLE_TH1_HOOKS)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_TH1_HOOKS=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_TH1_HOOKS=1
 !endif
 
-!ifdef FOSSIL_ENABLE_TCL
+!if $(FOSSIL_ENABLE_TCL)!=0
 TCC       = $(TCC) /DFOSSIL_ENABLE_TCL=1
 RCC       = $(RCC) /DFOSSIL_ENABLE_TCL=1
 TCC       = $(TCC) /DFOSSIL_ENABLE_TCL_STUBS=1
@@ -1448,6 +1698,11 @@ RCC       = $(RCC) /DFOSSIL_ENABLE_TCL_PRIVATE_STUBS=1
 TCC       = $(TCC) /DUSE_TCL_STUBS=1
 RCC       = $(RCC) /DUSE_TCL_STUBS=1
 !endif
+
+!if $(USE_SEE)!=0
+TCC       = $(TCC) /DUSE_SEE=1
+RCC       = $(RCC) /DUSE_SEE=1
+!endif
 }
 regsub -all {[-]D} [join $SQLITE_WIN32_OPTIONS { }] {/D} MSC_SQLITE_OPTIONS
 set j " \\\n                 "
@@ -1494,12 +1749,12 @@ foreach s [lsort [concat $src $AdditionalObj]] {
 if {$i > 0} {
   writeln " \\"
 }
-writeln "!ifdef FOSSIL_ENABLE_MINIZ"
+writeln "!if \$(FOSSIL_ENABLE_MINIZ)!=0"
 writeln -nonewline "        "
 writeln "\$(OX)\\miniz\$O \\"; incr i
 writeln "!endif"
 writeln -nonewline "        \$(OX)\\fossil.res\n\n"
-writeln {
+writeln [string map [list <<<NEXT_LINE>>> \\] {
 APPNAME    = $(OX)\fossil$(E)
 PDBNAME    = $(OX)\fossil$(P)
 APPTARGETS =
@@ -1508,13 +1763,13 @@ all: $(OX) $(APPNAME)
 
 zlib:
 	@echo Building zlib from "$(ZLIBDIR)"...
-!ifdef FOSSIL_ENABLE_WINXP
+!if $(FOSSIL_ENABLE_WINXP)!=0
 	@pushd "$(ZLIBDIR)" && $(MAKE) /f win32\Makefile.msc $(ZLIB) "CC=cl $(XPCFLAGS)" "LD=link $(XPLDFLAGS)" && popd
 !else
 	@pushd "$(ZLIBDIR)" && $(MAKE) /f win32\Makefile.msc $(ZLIB) && popd
 !endif
 
-!ifdef FOSSIL_ENABLE_SSL
+!if $(FOSSIL_ENABLE_SSL)!=0
 openssl:
 	@echo Building OpenSSL from "$(SSLDIR)"...
 !if "$(PERLDIR)" != ""
@@ -1522,19 +1777,21 @@ openssl:
 !endif
 	@pushd "$(SSLDIR)" && $(PERL) Configure $(SSLCONFIG) && popd
 	@pushd "$(SSLDIR)" && call $(SSLSETUP) && popd
-!ifdef FOSSIL_ENABLE_WINXP
+!if $(FOSSIL_ENABLE_WINXP)!=0
 	@pushd "$(SSLDIR)" && $(MAKE) /f $(SSLNMAKE) "CC=cl $(SSLCFLAGS) $(XPCFLAGS)" "LFLAGS=$(SSLLFLAGS) $(XPLDFLAGS)" && popd
 !else
 	@pushd "$(SSLDIR)" && $(MAKE) /f $(SSLNMAKE) "CC=cl $(SSLCFLAGS)" && popd
 !endif
 !endif
 
-!ifndef FOSSIL_ENABLE_MINIZ
+!if $(FOSSIL_ENABLE_MINIZ)==0
+!if $(FOSSIL_BUILD_ZLIB)!=0
 APPTARGETS = $(APPTARGETS) zlib
 !endif
+!endif
 
-!ifdef FOSSIL_ENABLE_SSL
-!ifdef FOSSIL_BUILD_SSL
+!if $(FOSSIL_ENABLE_SSL)!=0
+!if $(FOSSIL_BUILD_SSL)!=0
 APPTARGETS = $(APPTARGETS) openssl
 !endif
 !endif
@@ -1543,15 +1800,17 @@ $(APPNAME) : $(APPTARGETS) translate$E mkindex$E codecheck1$E headers $(OBJ) $(O
 	cd $(OX)
 	codecheck1$E $(SRC)
 	link $(LDFLAGS) /OUT:$@ $(LIBDIR) Wsetargv.obj fossil.res @linkopts
+	if exist $@.manifest <<<NEXT_LINE>>>
+		$(MTC) -nologo -manifest $@.manifest -outputresource:$@;1
 
-$(OX)\linkopts: $B\win\Makefile.msc}
+$(OX)\linkopts: $B\win\Makefile.msc}]
 set redir {>}
 foreach s [lsort [concat $src $AdditionalObj]] {
   writeln "\techo \$(OX)\\$s.obj $redir \$@"
   set redir {>>}
 }
 set redir {>>}
-writeln "!ifdef FOSSIL_ENABLE_MINIZ"
+writeln "!if \$(FOSSIL_ENABLE_MINIZ)!=0"
 writeln "\techo \$(OX)\\miniz.obj $redir \$@"
 writeln "!endif"
 writeln "\techo \$(LIBS) $redir \$@"
@@ -1577,11 +1836,23 @@ mkversion$E: $(SRCDIR)\mkversion.c
 codecheck1$E: $(SRCDIR)\codecheck1.c
 	$(BCC) $**
 
-$(OX)\shell$O : $(SRCDIR)\shell.c $B\win\Makefile.msc
-	$(TCC) /Fo$@ $(SHELL_OPTIONS) $(SQLITE_OPTIONS) $(SHELL_CFLAGS) -c $(SRCDIR)\shell.c
+!if $(USE_SEE)!=0
+SQLITE3_SHELL_SRC = $(SRCDIR)\shell-see.c
+!else
+SQLITE3_SHELL_SRC = $(SRCDIR)\shell.c
+!endif
 
-$(OX)\sqlite3$O : $(SRCDIR)\sqlite3.c $B\win\Makefile.msc
-	$(TCC) /Fo$@ -c $(SQLITE_OPTIONS) $(SQLITE_CFLAGS) $(SRCDIR)\sqlite3.c
+$(OX)\shell$O : $(SQLITE3_SHELL_SRC) $B\win\Makefile.msc
+	$(TCC) /Fo$@ $(SHELL_OPTIONS) $(SQLITE_OPTIONS) $(SHELL_CFLAGS) -c $(SQLITE3_SHELL_SRC)
+
+!if $(USE_SEE)!=0
+SQLITE3_SRC = $(SRCDIR)\sqlite3-see.c
+!else
+SQLITE3_SRC = $(SRCDIR)\sqlite3.c
+!endif
+
+$(OX)\sqlite3$O : $(SQLITE3_SRC) $B\win\Makefile.msc
+	$(TCC) /Fo$@ -c $(SQLITE_OPTIONS) $(SQLITE_CFLAGS) $(SEE_FLAGS) $(SQLITE3_SRC)
 
 $(OX)\th$O : $(SRCDIR)\th.c
 	$(TCC) /Fo$@ -c $**
