@@ -382,9 +382,9 @@ Manifest *manifest_parse(Blob *pContent, int rid, Blob *pErr){
   if( !isRepeat ) g.parseCnt[0]++;
   z = blob_materialize(pContent);
   n = blob_size(pContent);
-  if( pErr && (n<=0 || z[n-1]!='\n') ){
+  if( n<=0 || z[n-1]!='\n' ){
     blob_reset(pContent);
-    blob_append(pErr, n ? "not terminated with \\n" : "zero-length", -1);
+    blob_appendf(pErr, "%s", n ? "not terminated with \\n" : "zero-length");
     return 0;
   }
 
@@ -1187,6 +1187,11 @@ int manifest_file_mperm(ManifestFile *pFile){
 /*
 ** Add a single entry to the mlink table.  Also add the filename to
 ** the filename table if it is not there already.
+**
+** An mlink entry is always created if isPrimary is true.  But if
+** isPrimary is false (meaning that pmid is a merge parent of mid)
+** then the mlink entry is only created if there is already an mlink
+** from primary parent for the same file.
 */
 static void add_one_mlink(
   int pmid,                 /* The parent manifest */
@@ -1200,7 +1205,8 @@ static void add_one_mlink(
   int mperm                 /* 1: exec, 2: symlink */
 ){
   int fnid, pfnid, pid, fid;
-  static Stmt s1;
+  int doInsert;
+  static Stmt s1, s2;
 
   fnid = filename_to_fnid(zFilename);
   if( zPrior==0 ){
@@ -1219,19 +1225,32 @@ static void add_one_mlink(
     fid = uuid_to_rid(zToUuid, 1);
     if( isPublic ) content_make_public(fid);
   }
-  db_static_prepare(&s1,
-    "INSERT INTO mlink(mid,fid,pmid,pid,fnid,pfnid,mperm,isaux)"
-    "VALUES(:m,:f,:pm,:p,:n,:pfn,:mp,:isaux)"
-  );
-  db_bind_int(&s1, ":m", mid);
-  db_bind_int(&s1, ":f", fid);
-  db_bind_int(&s1, ":pm", pmid);
-  db_bind_int(&s1, ":p", pid);
-  db_bind_int(&s1, ":n", fnid);
-  db_bind_int(&s1, ":pfn", pfnid);
-  db_bind_int(&s1, ":mp", mperm);
-  db_bind_int(&s1, ":isaux", isPrimary==0);
-  db_exec(&s1);
+  if( isPrimary ){
+    doInsert = 1;
+  }else{
+    db_static_prepare(&s2,
+      "SELECT 1 FROM mlink WHERE mid=:m AND fnid=:n AND NOT isaux"
+    );
+    db_bind_int(&s2, ":m", mid);
+    db_bind_int(&s2, ":n", fnid);
+    doInsert = db_step(&s2)==SQLITE_ROW;
+    db_reset(&s2);
+  }
+  if( doInsert ){
+    db_static_prepare(&s1,
+      "INSERT INTO mlink(mid,fid,pmid,pid,fnid,pfnid,mperm,isaux)"
+      "VALUES(:m,:f,:pm,:p,:n,:pfn,:mp,:isaux)"
+    );
+    db_bind_int(&s1, ":m", mid);
+    db_bind_int(&s1, ":f", fid);
+    db_bind_int(&s1, ":pm", pmid);
+    db_bind_int(&s1, ":p", pid);
+    db_bind_int(&s1, ":n", fnid);
+    db_bind_int(&s1, ":pfn", pfnid);
+    db_bind_int(&s1, ":mp", mperm);
+    db_bind_int(&s1, ":isaux", isPrimary==0);
+    db_exec(&s1);
+  }
   if( pid && fid ){
     content_deltify(pid, fid, 0);
   }
@@ -1346,12 +1365,22 @@ ManifestFile *manifest_file_find(Manifest *p, const char *zName){
 **
 ** Deleted files have mlink.fid=0.
 ** Added files have mlink.pid=0.
+** File added by merge have mlink.pid=-1
 ** Edited files have both mlink.pid!=0 and mlink.fid!=0
+**
+** Many mlink entries for merge parents will only be added if another mlink
+** entry already exists for the same file from the primary parent.  Therefore,
+** to ensure that all merge-parent mlink entries are properly created:
+**
+**    (1) Make this routine a no-op if pParent is a merge parent and the
+**        primary parent is a phantom.
+**    (2) Invoke this routine recursively for merge-parents if pParent is the
+**        primary parent.
 */
 static void add_mlink(
-  int pmid, Manifest *pParent,    /* Parent check-in */
-  int mid,  Manifest *pChild,     /* The child check-in */
-  int isPrim                      /* TRUE if pmid is the primary parent of mid */
+  int pmid, Manifest *pParent,  /* Parent check-in */
+  int mid,  Manifest *pChild,   /* The child check-in */
+  int isPrim                    /* TRUE if pmid is the primary parent of mid */
 ){
   Blob otherContent;
   int otherRid;
@@ -1393,6 +1422,19 @@ static void add_mlink(
     return;
   }
   isPublic = !content_is_private(mid);
+  
+  /* If pParent is not the primary parent of pChild, and the primary
+  ** parent of pChild is a phantom, then abort this routine without
+  ** doing any work.  The mlink entries will be computed when the
+  ** primary parent dephantomizes.
+  */
+  if( !isPrim && otherRid==mid
+   && !db_exists("SELECT 1 FROM blob WHERE uuid=%Q AND size>0",
+                 pChild->azParent[0])
+  ){
+    manifest_cache_insert(*ppOther);
+    return;
+  }
 
   /* Try to make the parent manifest a delta from the child, if that
   ** is an appropriate thing to do.  For a new baseline, make the
@@ -1490,11 +1532,134 @@ static void add_mlink(
     }
   }
   manifest_cache_insert(*ppOther);
+  
+  /* If pParent is the primary parent of pChild, also run this analysis
+  ** for all merge parents of pChild
+  */
+  if( isPrim ){
+    for(i=1; i<pChild->nParent; i++){
+      pmid = uuid_to_rid(pChild->azParent[i], 0);
+      if( pmid<=0 ) continue;
+      add_mlink(pmid, 0, mid, pChild, 0);
+    }
+  }
+}
+
+/*
+** For a check-in with RID "rid" that has nParent parent check-ins given
+** by the UUIDs in azParent[], create all appropriate plink and mlink table
+** entries.
+**
+** The primary parent is the first UUID on the azParent[] list.
+**
+** Return the RID of the primary parent.
+*/
+static int manifest_add_checkin_linkages(
+  int rid,                   /* The RID of the check-in */
+  Manifest *p,               /* Manifest for this check-in */
+  int nParent,               /* Number of parents for this check-in */
+  char **azParent            /* UUIDs for each parent */
+){
+  int i;
+  int parentid = 0;
+  char zBaseId[30];    /* Baseline manifest RID for deltas.  "NULL" otherwise */
+  Stmt q;
+
+  if( p->zBaseline ){
+     sqlite3_snprintf(sizeof(zBaseId), zBaseId, "%d", 
+                      uuid_to_rid(p->zBaseline,1));
+  }else{
+     sqlite3_snprintf(sizeof(zBaseId), zBaseId, "NULL");
+  }
+  for(i=0; i<nParent; i++){
+    int pid = uuid_to_rid(azParent[i], 1);
+    db_multi_exec(
+       "INSERT OR IGNORE INTO plink(pid, cid, isprim, mtime, baseid)"
+        "VALUES(%d, %d, %d, %.17g, %s)",
+       pid, rid, i==0, p->rDate, zBaseId/*safe-for-%s*/);
+    if( i==0 ) parentid = pid;
+  }
+  add_mlink(parentid, 0, rid, p, 1);
+  if( nParent>1 ){
+    /* Change MLINK.PID from 0 to -1 for files that are added by merge. */
+    db_multi_exec(
+      "UPDATE mlink SET pid=-1"
+      " WHERE mid=%d"
+      "   AND pid=0"
+      "   AND fnid IN "
+      "  (SELECT fnid FROM mlink WHERE mid=%d GROUP BY fnid"
+      "    HAVING count(*)<%d)",
+      rid, rid, nParent
+    );
+  }
+  db_prepare(&q, "SELECT cid, isprim FROM plink WHERE pid=%d", rid);
+  while( db_step(&q)==SQLITE_ROW ){
+    int cid = db_column_int(&q, 0);
+    int isprim = db_column_int(&q, 1);
+    add_mlink(rid, p, cid, 0, isprim);
+  }
+  db_finalize(&q);
+  if( nParent==0 ){
+    /* For root files (files without parents) add mlink entries
+    ** showing all content as new. */
+    int isPublic = !content_is_private(rid);
+    for(i=0; i<p->nFile; i++){
+      add_one_mlink(0, 0, rid, p->aFile[i].zUuid, p->aFile[i].zName, 0,
+                    isPublic, 1, manifest_file_mperm(&p->aFile[i]));
+    }
+  }
+  return parentid;
+}
+
+/*
+** There exists a "parent" tag against checkin rid that has value zValue.
+** If value is well-formed (meaning that it is a list of UUIDs), then use 
+** zValue to reparent check-in rid.
+*/
+void manifest_reparent_checkin(int rid, const char *zValue){
+  int nParent;
+  char *zCopy = 0;
+  char **azParent = 0;
+  Manifest *p = 0;
+  int i;
+  int n = (int)strlen(zValue);
+  nParent = (n+1)/(UUID_SIZE+1);
+  if( nParent*(UUID_SIZE+1) - 1 !=n ) return;
+  if( nParent<1 ) return;
+  zCopy = fossil_strdup(zValue);
+  azParent = fossil_malloc( sizeof(azParent[0])*nParent );
+  for(i=0; i<nParent; i++){
+    azParent[i] = &zCopy[i*(UUID_SIZE+1)];
+    if( i<nParent-1 && azParent[i][UUID_SIZE]!=' ' ) break;
+    azParent[i][UUID_SIZE] = 0;
+    if( !validate16(azParent[i],UUID_SIZE) ) break;
+  }
+  if( i==nParent 
+   && !db_exists("SELECT 1 FROM plink WHERE cid=%d AND pid=%d",
+                 rid, uuid_to_rid(azParent[0],0))
+  ){
+    p = manifest_get(rid, CFTYPE_MANIFEST, 0);
+  }
+  if( p!=0 ){
+    db_multi_exec(
+       "DELETE FROM plink WHERE cid=%d;"
+       "DELETE FROM mlink WHERE mid=%d;",
+       rid, rid
+    );
+    manifest_add_checkin_linkages(rid,p,nParent,azParent);
+  }
+  manifest_destroy(p);
+  fossil_free(azParent);
+  fossil_free(zCopy);
 }
 
 /*
 ** Setup to do multiple manifest_crosslink() calls.
-** This is only required if processing ticket changes.
+**
+** This routine creates TEMP tables for holding information for
+** processing that must be deferred until all artifacts have been
+** seen at least once.  The deferred processing is accomplished
+** by the call to manifest_crosslink_end().
 */
 void manifest_crosslink_begin(void){
   assert( manifest_crosslink_busy==0 );
@@ -1541,6 +1706,17 @@ int manifest_crosslink_end(int flags){
       zScript = xfer_ticket_code();
     }
   }
+  db_prepare(&q, 
+     "SELECT rid, value FROM tagxref"
+     " WHERE tagid=%d AND tagtype=1",
+     TAG_PARENT
+  );
+  while( db_step(&q)==SQLITE_ROW ){
+    int rid = db_column_int(&q,0);
+    const char *zValue = db_column_text(&q,1);
+    manifest_reparent_checkin(rid, zValue);
+  }
+  db_finalize(&q);
   db_prepare(&q, "SELECT uuid FROM pending_tkt");
   while( db_step(&q)==SQLITE_ROW ){
     const char *zUuid = db_column_text(&q, 0);
@@ -1786,51 +1962,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     }
     if( !db_exists("SELECT 1 FROM mlink WHERE mid=%d", rid) ){
       char *zCom;
-      char zBaseId[30];
-      if( p->zBaseline ){
-        sqlite3_snprintf(sizeof(zBaseId), zBaseId, "%d",
-                         uuid_to_rid(p->zBaseline,1));
-      }else{
-        sqlite3_snprintf(sizeof(zBaseId), zBaseId, "NULL");
-      }
-      for(i=0; i<p->nParent; i++){
-        int pid = uuid_to_rid(p->azParent[i], 1);
-        db_multi_exec(
-           "INSERT OR IGNORE INTO plink(pid, cid, isprim, mtime, baseid)"
-           "VALUES(%d, %d, %d, %.17g, %s)",
-           pid, rid, i==0, p->rDate, zBaseId/*safe-for-%s*/);
-        add_mlink(pid, 0, rid, p, i==0);
-        if( i==0 ) parentid = pid;
-      }
-      if( p->nParent>1 ){
-        /* Remove incorrect MLINK create-file entries that arise when a
-        ** file is added by merge. */
-        db_multi_exec(
-           "DELETE FROM mlink"
-           " WHERE mid=%d"
-           "   AND pid=0"
-           "   AND fnid IN "
-           "  (SELECT fnid FROM mlink WHERE mid=%d GROUP BY fnid"
-           "    HAVING count(*)<%d)",
-           rid, rid, p->nParent
-        );
-      }
-      db_prepare(&q, "SELECT cid, isprim FROM plink WHERE pid=%d", rid);
-      while( db_step(&q)==SQLITE_ROW ){
-        int cid = db_column_int(&q, 0);
-        int isprim = db_column_int(&q, 1);
-        add_mlink(rid, p, cid, 0, isprim);
-      }
-      db_finalize(&q);
-      if( p->nParent==0 ){
-        /* For root files (files without parents) add mlink entries
-        ** showing all content as new. */
-        int isPublic = !content_is_private(rid);
-        for(i=0; i<p->nFile; i++){
-          add_one_mlink(0, 0, rid, p->aFile[i].zUuid, p->aFile[i].zName, 0,
-                        isPublic, 1, manifest_file_mperm(&p->aFile[i]));
-        }
-      }
+      parentid = manifest_add_checkin_linkages(rid,p,p->nParent,p->azParent);
       search_doc_touch('c', rid, 0);
       db_multi_exec(
         "REPLACE INTO event(type,mtime,objid,user,comment,"
@@ -1958,6 +2090,7 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
     int prior, subsequent;
     int nWiki;
     char zLength[40];
+    Stmt qatt;
     while( fossil_isspace(p->zWiki[0]) ) p->zWiki++;
     nWiki = strlen(p->zWiki);
     sqlite3_snprintf(sizeof(zLength), zLength, "%d", nWiki);
@@ -1999,22 +2132,93 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
         TAG_BGCOLOR, rid
       );
     }
+    /* Locate and update comment for any attachments */
+    db_prepare(&qatt,
+       "SELECT attachid, src, target, filename FROM attachment"
+       " WHERE target=%Q",
+       p->zEventId
+    );
+    while( db_step(&qatt)==SQLITE_ROW ){
+      const char *zAttachId = db_column_text(&qatt, 0);
+      const char *zSrc = db_column_text(&qatt, 1);
+      const char *zTarget = db_column_text(&qatt, 2);
+      const char *zName = db_column_text(&qatt, 3);
+      const char isAdd = (zSrc && zSrc[0]) ? 1 : 0;
+      char *zComment;
+      if( isAdd ){
+        zComment = mprintf(
+             "Add attachment [/artifact/%!S|%h] to"
+             " tech note [/technote/%!S|%S]",
+             zSrc, zName, zTarget, zTarget); 
+      }else{
+        zComment = mprintf(
+             "Delete attachment \"%h\" from"
+             " tech note [/technote/%!S|%S]",
+             zName, zTarget, zTarget);
+      }
+      db_multi_exec("UPDATE event SET comment=%Q, type='e'"
+                       " WHERE objid=%Q",
+                    zComment, zAttachId);
+      fossil_free(zComment);      
+    }
+    db_finalize(&qatt);
   }
   if( p->type==CFTYPE_TICKET ){
     char *zTag;
+    Stmt qatt;
     assert( manifest_crosslink_busy==1 );
     zTag = mprintf("tkt-%s", p->zTicketUuid);
     tag_insert(zTag, 1, 0, rid, p->rDate, rid);
     fossil_free(zTag);
     db_multi_exec("INSERT OR IGNORE INTO pending_tkt VALUES(%Q)",
                   p->zTicketUuid);
+    /* Locate and update comment for any attachments */
+    db_prepare(&qatt,
+       "SELECT attachid, src, target, filename FROM attachment"
+       " WHERE target=%Q",
+       p->zTicketUuid
+    );
+    while( db_step(&qatt)==SQLITE_ROW ){
+      const char *zAttachId = db_column_text(&qatt, 0);
+      const char *zSrc = db_column_text(&qatt, 1);
+      const char *zTarget = db_column_text(&qatt, 2);
+      const char *zName = db_column_text(&qatt, 3);
+      const char isAdd = (zSrc && zSrc[0]) ? 1 : 0;
+      char *zComment;
+      if( isAdd ){
+        zComment = mprintf(
+             "Add attachment [/artifact/%!S|%h] to ticket [%!S|%S]",
+             zSrc, zName, zTarget, zTarget);
+      }else{
+        zComment = mprintf("Delete attachment \"%h\" from ticket [%!S|%S]",
+             zName, zTarget, zTarget);
+      }
+      db_multi_exec("UPDATE event SET comment=%Q, type='t'"
+                       " WHERE objid=%Q",
+                    zComment, zAttachId);
+      fossil_free(zComment);      
+    }
+    db_finalize(&qatt);
   }
   if( p->type==CFTYPE_ATTACHMENT ){
     char *zComment = 0;
     const char isAdd = (p->zAttachSrc && p->zAttachSrc[0]) ? 1 : 0;
-    const char attachToType = fossil_is_uuid(p->zAttachTarget)
-      ? 't' /* attach to ticket */
-      : 'w' /* attach to wiki page */;
+    /* We assume that we're attaching to a wiki page until we
+    ** prove otherwise (which could on a later artifact if we
+    ** process the attachment artifact before the artifact to
+    ** which it is attached!) */
+    char attachToType = 'w';       
+    if( fossil_is_uuid(p->zAttachTarget) ){
+      if( db_exists("SELECT 1 FROM tag WHERE tagname='tkt-%q'",
+            p->zAttachTarget)
+        ){
+        attachToType = 't';          /* Attaching to known ticket */
+      }else if( db_exists("SELECT 1 FROM tag WHERE tagname='event-%q'",
+                  p->zAttachTarget) 
+            ){
+        attachToType = 'e';          /* Attaching to known tech note */
+      }
+    }
     db_multi_exec(
        "INSERT INTO attachment(attachid, mtime, src, target,"
                               "filename, comment, user)"
@@ -2039,6 +2243,18 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
         zComment = mprintf("Delete attachment \"%h\" from wiki page [%h]",
              p->zAttachName, p->zAttachTarget);
       }
+    }else if( 'e' == attachToType ){
+      if( isAdd ){
+        zComment = mprintf(
+          "Add attachment [/artifact/%!S|%h] to tech note [/technote/%!S|%S]",
+          p->zAttachSrc, p->zAttachName, p->zAttachTarget, p->zAttachTarget); 
+      }else{
+        zComment = mprintf(
+             "Delete attachment \"/artifact/%!S|%h\" from"
+             " tech note [/technote/%!S|%S]",
+             p->zAttachName, p->zAttachName,
+             p->zAttachTarget,p->zAttachTarget);
+      }      
     }else{
       if( isAdd ){
         zComment = mprintf(
@@ -2112,7 +2328,9 @@ int manifest_crosslink(int rid, Blob *pContent, int flags){
         blob_appendf(&comment, " Timestamp %h.", zValue);
         continue;
       }else if( memcmp(zName, "-sym-",5)==0 ){
-        if( !branchMove ) blob_appendf(&comment, " Cancel tag \"%h\"", &zName[5]);
+        if( !branchMove ){
+          blob_appendf(&comment, " Cancel tag \"%h\"", &zName[5]);
+        }
       }else if( memcmp(zName, "*sym-",5)==0 ){
         if( !branchMove ){
           blob_appendf(&comment, " Add propagating tag \"%h\"", &zName[5]);
