@@ -524,27 +524,29 @@ static void reconstruct_private_table(void){
 ** executable in a way that changes the database schema.
 **
 ** Options:
-**   --analyze     Run ANALYZE on the database after rebuilding
-**   --cluster     Compute clusters for unclustered artifacts
-**   --compress    Strive to make the database as small as possible
-**   --deanalyze   Remove ANALYZE tables from the database
-**   --force       Force the rebuild to complete even if errors are seen
-**   --ifneeded    Only do the rebuild if it would change the schema version
-**   --index       Always add in the full-text search index
-**   --noverify    Skip the verification of changes to the BLOB table
-**   --noindex     Always omit the full-text search index
-**   --pagesize N  Set the database pagesize to N. (512..65536 and power of 2)
-**   --randomize   Scan artifacts in a random order
-**   --stats       Show artifact statistics after rebuilding
-**   --vacuum      Run VACUUM on the database after rebuilding
-**   --wal         Set Write-Ahead-Log journalling mode on the database
+**   --analyze         Run ANALYZE on the database after rebuilding
+**   --cluster         Compute clusters for unclustered artifacts
+**   --compress        Strive to make the database as small as possible
+**   --compress-only   Skip the rebuilding step. Do --compress only
+**   --deanalyze       Remove ANALYZE tables from the database
+**   --force           Force the rebuild to complete even if errors are seen
+**   --ifneeded        Only do the rebuild if it would change the schema version
+**   --index           Always add in the full-text search index
+**   --noverify        Skip the verification of changes to the BLOB table
+**   --noindex         Always omit the full-text search index
+**   --pagesize N      Set the database pagesize to N. (512..65536 and power of 2)
+**   --quiet           Only show output if there are errors
+**   --randomize       Scan artifacts in a random order
+**   --stats           Show artifact statistics after rebuilding
+**   --vacuum          Run VACUUM on the database after rebuilding
+**   --wal             Set Write-Ahead-Log journalling mode on the database
 **
 ** See also: deconstruct, reconstruct
 */
 void rebuild_database(void){
   int forceFlag;
   int randomizeFlag;
-  int errCnt;
+  int errCnt = 0;
   int omitVerify;
   int doClustering;
   const char *zPagesize;
@@ -559,6 +561,7 @@ void rebuild_database(void){
   int optNoIndex;
   int optIndex;
   int optIfNeeded;
+  int compressOnlyFlag;
 
   omitVerify = find_option("noverify",0,0)!=0;
   forceFlag = find_option("force","f",0)!=0;
@@ -573,6 +576,8 @@ void rebuild_database(void){
   optIndex = find_option("index",0,0)!=0;
   optNoIndex = find_option("noindex",0,0)!=0;
   optIfNeeded = find_option("ifneeded",0,0)!=0;
+  compressOnlyFlag = find_option("compress-only",0,0)!=0;
+  if( compressOnlyFlag ) runCompress = runVacuum = 1;
   if( zPagesize ){
     newPagesize = atoi(zPagesize);
     if( newPagesize<512 || newPagesize>65536
@@ -592,7 +597,7 @@ void rebuild_database(void){
     db_close(1);
     db_open_repository(g.zRepositoryName);
   }
-  runReindex = search_index_exists();
+  runReindex = search_index_exists() && !compressOnlyFlag;
   if( optIndex ) runReindex = 1;
   if( optNoIndex ) runReindex = 0;
   if( optIfNeeded && fossil_strcmp(db_get("aux-schema",""),AUX_SCHEMA_MAX)==0 ){
@@ -603,10 +608,12 @@ void rebuild_database(void){
   verify_all_options();
 
   db_begin_transaction();
-  search_drop_index();
-  ttyOutput = 1;
-  errCnt = rebuild_db(randomizeFlag, 1, doClustering);
-  reconstruct_private_table();
+  if( !compressOnlyFlag ){
+    search_drop_index();
+    ttyOutput = 1;
+    errCnt = rebuild_db(randomizeFlag, 1, doClustering);
+    reconstruct_private_table();
+  }
   db_multi_exec(
     "REPLACE INTO config(name,value,mtime) VALUES('content-schema',%Q,now());"
     "REPLACE INTO config(name,value,mtime) VALUES('aux-schema',%Q,now());"
@@ -677,7 +684,9 @@ void rebuild_database(void){
 }
 
 /*
-** COMMAND:  test-detach  ?REPOSITORY?
+** COMMAND: test-detach
+** 
+** Usage: %fossil test-detach  ?REPOSITORY?
 **
 ** Change the project-code and make other changes in order to prevent
 ** the repository from ever again pushing or pulling to other
@@ -698,7 +707,7 @@ void test_detach_cmd(void){
 }
 
 /*
-** COMMAND:  test-create-clusters
+** COMMAND: test-create-clusters
 **
 ** Create clusters for all unclustered artifacts if the number of unclustered
 ** artifacts exceeds the current clustering threshold.
@@ -789,7 +798,8 @@ void test_clusters_cmd(void){
 
 /*
 ** COMMAND: scrub*
-** %fossil scrub ?OPTIONS? ?REPOSITORY?
+** 
+** Usage: %fossil scrub ?OPTIONS? ?REPOSITORY?
 **
 ** The command removes sensitive information (such as passwords) from a
 ** repository so that the repository can be sent to an untrusted reader.
@@ -882,7 +892,7 @@ void recon_read_dir(char *zPath){
   void *zUnicodePath;
   char *zUtf8Name;
 
-  zUnicodePath = fossil_utf8_to_filename(zPath);
+  zUnicodePath = fossil_utf8_to_path(zPath, 1);
   d = opendir(zUnicodePath);
   if( d ){
     while( (pEntry=readdir(d))!=0 ){
@@ -892,9 +902,9 @@ void recon_read_dir(char *zPath){
       if( pEntry->d_name[0]=='.' ){
         continue;
       }
-      zUtf8Name = fossil_filename_to_utf8(pEntry->d_name);
+      zUtf8Name = fossil_path_to_utf8(pEntry->d_name);
       zSubpath = mprintf("%s/%s", zPath, zUtf8Name);
-      fossil_filename_free(zUtf8Name);
+      fossil_path_free(zUtf8Name);
 #ifdef _DIRENT_HAVE_D_TYPE
       if( (pEntry->d_type==DT_UNKNOWN || pEntry->d_type==DT_LNK)
           ? (file_isdir(zSubpath)==1) : (pEntry->d_type==DT_DIR) )
@@ -923,7 +933,7 @@ void recon_read_dir(char *zPath){
     fossil_fatal("encountered error %d while trying to open \"%s\".",
                   errno, g.argv[3]);
   }
-  fossil_filename_free(zUnicodePath);
+  fossil_path_free(zUnicodePath);
 }
 
 /*
@@ -953,9 +963,9 @@ void reconstruct_cmd(void) {
   /* We should be done with options.. */
   verify_all_options();
 
-  db_open_config(0);
+  db_open_config(0, 0);
   db_begin_transaction();
-  db_initial_setup(0, 0, 0, 1);
+  db_initial_setup(0, 0, 0);
 
   fossil_print("Reading files from directory \"%s\"...\n", g.argv[3]);
   recon_read_dir(g.argv[3]);
