@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 2013 Stephen Beal
+** Copyright (c) 2013 Stephan Beal
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the Simplified BSD License (also
@@ -38,20 +38,6 @@ static int statsReportType = 0;
 static const char *statsReportTimelineYFlag = NULL;
 
 
-/*
-** Generate a submenu element with a single parameter change.
-*/
-static void statrep_submenu(
-  HQuery *pUrl,            /* Base URL */
-  const char *zMenuName,   /* Submenu name */
-  const char *zParam,      /* Parameter value to add or change */
-  const char *zValue,      /* Value of the new parameter */
-  const char *zRemove      /* Parameter to omit */
-){
-  style_submenu_element(zMenuName, zMenuName, "%s",
-                        url_render(pUrl, zParam, zValue, zRemove, 0));
-}
-
 /*
 ** Creates a TEMP VIEW named v_reports which is a wrapper around the
 ** EVENT table filtered on event.type. It looks for the request
@@ -143,59 +129,6 @@ static const char *stats_report_label_for_type(){
   }
 }
 
-/*
-** A helper for the /reports family of pages which prints out a menu
-** of links for the various type=XXX flags. zCurrentViewName must be
-** the name/value of the 'view' parameter which is in effect at the
-** time this is called. e.g. if called from the 'byuser' view then
-** zCurrentViewName must be "byuser". Any URL parameters which need to
-** be added to the generated URLs should be passed in zParam. The
-** caller is expected to have already encoded any zParam in the %T or
-** %t encoding.  */
-static void stats_report_event_types_menu(const char *zCurrentViewName,
-                                          const char *zParam){
-  char *zTop;
-  if(zParam && !*zParam){
-    zParam = NULL;
-  }
-  zTop = mprintf("%s/reports?view=%s%s%s", g.zTop, zCurrentViewName,
-                 zParam ? "&" : "", zParam);
-  cgi_printf("<div>");
-  cgi_printf("<span>Types:</span> ");
-  if('*' == statsReportType){
-    cgi_printf(" <strong>all</strong>", zTop);
-  }else{
-    cgi_printf(" <a href='%s'>all</a>", zTop);
-  }
-  if('c' == statsReportType){
-    cgi_printf(" <strong>check-ins</strong>", zTop);
-  }else{
-    cgi_printf(" <a href='%s&type=ci'>check-ins</a>", zTop);
-  }
-  if('e' == statsReportType){
-    cgi_printf(" <strong>technotes</strong>", zTop);
-  }else{
-    cgi_printf(" <a href='%s&type=e'>technotes</a>", zTop);
-  }
-  if( 't' == statsReportType ){
-    cgi_printf(" <strong>tickets</strong>", zTop);
-  }else{
-    cgi_printf(" <a href='%s&type=t'>tickets</a>", zTop);
-  }
-  if( 'g' == statsReportType ){
-    cgi_printf(" <strong>tags</strong>", zTop);
-  }else{
-    cgi_printf(" <a href='%s&type=g'>tags</a>", zTop);
-  }
-  if( 'w' == statsReportType ){
-    cgi_printf(" <strong>wiki</strong>", zTop);
-  }else{
-    cgi_printf(" <a href='%s&type=w'>wiki</a>", zTop);
-  }
-  fossil_free(zTop);
-  cgi_printf("</div>");
-}
-
 
 /*
 ** Helper for stats_report_by_month_year(), which generates a list of
@@ -229,9 +162,8 @@ static void stats_report_output_week_links(const char *zTimeframe){
 /*
 ** Implements the "byyear" and "bymonth" reports for /reports.
 ** If includeMonth is true then it generates the "bymonth" report,
-** else the "byyear" report. If zUserName is not NULL and not empty
-** then the report is restricted to events created by the named user
-** account.
+** else the "byyear" report. If zUserName is not NULL then the report is
+** restricted to events created by the named user account.
 */
 static void stats_report_by_month_year(char includeMonth,
                                        char includeWeeks,
@@ -241,7 +173,6 @@ static void stats_report_by_month_year(char includeMonth,
   int nEventTotal = 0;               /* Total event count */
   int rowClass = 0;                  /* counter for alternating
                                         row colors */
-  Blob sql = empty_blob;             /* SQL */
   const char *zTimeLabel = includeMonth ? "Year/Month" : "Year";
   char zPrevYear[5] = {0};           /* For keeping track of when
                                         we change years while looping */
@@ -249,32 +180,30 @@ static void stats_report_by_month_year(char includeMonth,
                                         current year */
   char showYearTotal = 0;            /* Flag telling us when to show
                                         the per-year event totals */
-  Blob header = empty_blob;          /* Page header text */
   int nMaxEvents  = 1;               /* for calculating length of graph
                                         bars. */
   int iterations = 0;                /* number of weeks/months we iterate
                                         over */
+  Blob userFilter = empty_blob;      /* Optional user=johndoe query string */
   stats_report_init_view();
-  stats_report_event_types_menu( includeMonth ? "bymonth" : "byyear", NULL );
-  blob_appendf(&header, "Timeline Events (%s) by year%s",
-               stats_report_label_for_type(),
-               (includeMonth ? "/month" : ""));
-  blob_append_sql(&sql,
-               "SELECT substr(date(mtime),1,%d) AS timeframe, "
-               "count(*) AS eventCount "
-               "FROM v_reports ",
-               includeMonth ? 7 : 4);
-  if(zUserName&&*zUserName){
-    blob_append_sql(&sql, " WHERE user=%Q ", zUserName);
-    blob_appendf(&header," for user %q", zUserName);
+  if( zUserName ){
+    blob_appendf(&userFilter, "user=%s", zUserName);
   }
-  blob_append(&sql,
-              " GROUP BY timeframe"
-              " ORDER BY timeframe DESC",
-              -1);
-  db_prepare(&query, "%s", blob_sql_text(&sql));
-  blob_reset(&sql);
-  @ <h1>%b(&header)</h1>
+  blob_reset(&userFilter);
+  db_prepare(&query,
+             "SELECT substr(date(mtime),1,%d) AS timeframe,"
+             "       count(*) AS eventCount"
+             "  FROM v_reports"
+             " WHERE ifnull(coalesce(euser,user,'')=%Q,1)"
+             " GROUP BY timeframe"
+             " ORDER BY timeframe DESC",
+             includeMonth ? 7 : 4, zUserName);
+  @ <h1>Timeline Events (%s(stats_report_label_for_type()))
+  @ by year%s(includeMonth ? "/month" : "")
+  if( zUserName ){
+    @ for user %h(zUserName)
+  }
+  @ </h1>
   @ <table class='statistics-report-table-events' border='0' cellpadding='2'
   @  cellspacing='0' id='statsTable'>
   @ <thead>
@@ -282,7 +211,6 @@ static void stats_report_by_month_year(char includeMonth,
   @ <th>Events</th>
   @ <th width='90%%'><!-- relative commits graph --></th>
   @ </thead><tbody>
-  blob_reset(&header);
   /*
      Run the query twice. The first time we calculate the maximum
      number of events for a given row. Maybe someone with better SQL
@@ -315,6 +243,7 @@ static void stats_report_by_month_year(char includeMonth,
           @ <td></td>
           @ <td colspan='2'>Yearly total: %d(nEventsPerYear)</td>
           @</tr>
+          showYearTotal = 0;
         }
         nEventsPerYear = 0;
         memcpy(zPrevYear,zTimeframe,4);
@@ -337,14 +266,14 @@ static void stats_report_by_month_year(char includeMonth,
       /* Reminder: n=nCount is not actually correct for bymonth unless
          that was the only user who caused events.
       */
-      if( zUserName && *zUserName ){
+      if( zUserName ){
         cgi_printf("&u=%t", zUserName);
       }
       cgi_printf("' target='_new'>%s</a>",zTimeframe);
     }else {
       cgi_printf("<a href='?view=byweek&y=%s&type=%c",
                  zTimeframe, (char)statsReportType);
-      if(zUserName && *zUserName){
+      if( zUserName ){
         cgi_printf("&u=%t", zUserName);
       }
       cgi_printf("'>%s</a>", zTimeframe);
@@ -405,14 +334,18 @@ static void stats_report_by_user(){
   int nMaxEvents = 1;                /* max number of events for
                                         all rows. */
   stats_report_init_view();
-  stats_report_event_types_menu("byuser", NULL);
-  db_prepare(&query,
-               "SELECT user, "
-               "COUNT(*) AS eventCount "
-               "FROM v_reports "
-               "GROUP BY user ORDER BY eventCount DESC");
   @ <h1>Timeline Events
   @ (%s(stats_report_label_for_type())) by User</h1>
+  db_multi_exec(
+    "CREATE TEMP VIEW piechart(amt,label) AS"
+    " SELECT count(*), ifnull(euser,user) FROM v_reports"
+                         " GROUP BY ifnull(euser,user) ORDER BY count(*) DESC;"
+  );
+  if( db_int(0, "SELECT count(*) FROM piechart")>=2 ){
+    @ <center><svg width=700 height=400>
+    piechart_render(700, 400, PIE_OTHER|PIE_PERCENT);
+    @ </svg></centre><hr/>
+  }
   @ <table class='statistics-report-table-events' border='0'
   @ cellpadding='2' cellspacing='0' id='statsTable'>
   @ <thead><tr>
@@ -420,6 +353,11 @@ static void stats_report_by_user(){
   @ <th>Events</th>
   @ <th width='90%%'><!-- relative commits graph --></th>
   @ </tr></thead><tbody>
+  db_prepare(&query,
+               "SELECT ifnull(euser,user), "
+               "COUNT(*) AS eventCount "
+               "FROM v_reports "
+               "GROUP BY ifnull(euser,user) ORDER BY eventCount DESC");
   while( SQLITE_ROW == db_step(&query) ){
     const int nCount = db_column_int(&query, 1);
     if(nCount>nMaxEvents){
@@ -430,6 +368,7 @@ static void stats_report_by_user(){
   while( SQLITE_ROW == db_step(&query) ){
     const char *zUser = db_column_text(&query, 0);
     const int nCount = db_column_int(&query, 1);
+    char y = (char)statsReportType;
     int nSize = nCount
       ? (int)(100 * nCount / nMaxEvents)
       : 0;
@@ -437,9 +376,9 @@ static void stats_report_by_user(){
     else if(!nSize) nSize = 1;
     rowClass = ++nRowNumber % 2;
     nEventTotal += nCount;
-    @<tr class='row%d(rowClass)'>
+    @ <tr class='row%d(rowClass)'>
     @ <td>
-    @ <a href="?view=bymonth&user=%h(zUser)&type=%c((char)statsReportType)">%h(zUser)</a>
+    @ <a href="?view=bymonth&user=%h(zUser)&type=%c(y)">%h(zUser)</a>
     @ </td><td data-sortkey='%08x(-nCount)'>%d(nCount)</td>
     @ <td>
     @ <div class='statistics-report-graph-line'
@@ -457,9 +396,10 @@ static void stats_report_by_user(){
 }
 
 /*
-** Implements the "byfile" view for /reports.
+** Implements the "byfile" view for /reports. If zUserName is not NULL then the
+** report is restricted to events created by the named user account.
 */
-static void stats_report_by_file(){
+static void stats_report_by_file(const char *zUserName){
   Stmt query;
   int mxEvent = 1;       /* max number of events across all rows */
   int nRowNumber = 0;
@@ -468,15 +408,21 @@ static void stats_report_by_file(){
     "CREATE TEMP TABLE statrep(filename, cnt);"
     "INSERT INTO statrep(filename, cnt)"
     "  SELECT filename.name, count(distinct mlink.mid)"
-    "    FROM filename, mlink"
+    "    FROM filename, mlink, event"
     "   WHERE filename.fnid=mlink.fnid"
-    "   GROUP BY 1"
+    "     AND mlink.mid=event.objid"
+    "     AND ifnull(coalesce(euser,user,'')=%Q,1)"
+    "   GROUP BY 1", zUserName
   );
   db_prepare(&query,
     "SELECT filename, cnt FROM statrep ORDER BY cnt DESC, filename /*sort*/"
   );
   mxEvent = db_int(1, "SELECT max(cnt) FROM statrep");
-  @ <h1>Check-ins Per File</h1>
+  @ <h1>Check-ins Per File
+  if( zUserName ){
+    @ for user %h(zUserName)
+  }
+  @ </h1>
   @ <table class='statistics-report-table-events' border='0'
   @ cellpadding='2' cellspacing='0' id='statsTable'>
   @ <thead><tr>
@@ -506,9 +452,10 @@ static void stats_report_by_file(){
 }
 
 /*
-** Implements the "byweekday" view for /reports.
+** Implements the "byweekday" view for /reports. If zUserName is not NULL then
+** the report is restricted to events created by the named user account.
 */
-static void stats_report_day_of_week(){
+static void stats_report_day_of_week(const char *zUserName){
   Stmt query = empty_Stmt;
   int nRowNumber = 0;                /* current TR number */
   int nEventTotal = 0;               /* Total event count */
@@ -516,20 +463,50 @@ static void stats_report_day_of_week(){
                                         row colors */
   int nMaxEvents = 1;                /* max number of events for
                                         all rows. */
+  Blob userFilter = empty_blob;      /* Optional user=johndoe query string */
   static const char *const daysOfWeek[] = {
-  "Monday", "Tuesday", "Wednesday", "Thursday",
-  "Friday", "Saturday", "Sunday"
+  "Sunday", "Monday", "Tuesday", "Wednesday",
+  "Thursday", "Friday", "Saturday"
   };
 
   stats_report_init_view();
-  stats_report_event_types_menu("byweekday", NULL);
+  if( zUserName ){
+    blob_appendf(&userFilter, "user=%s", zUserName);
+  }
   db_prepare(&query,
-               "SELECT cast(mtime %% 7 AS INTEGER) dow, "
-               "COUNT(*) AS eventCount "
-               "FROM v_reports "
-               "GROUP BY dow ORDER BY dow");
-  @ <h1>Timeline Events
-  @ (%s(stats_report_label_for_type())) by Day of the Week</h1>
+               "SELECT cast(strftime('%%w', mtime) AS INTEGER) dow,"
+               "       COUNT(*) AS eventCount"
+               "  FROM v_reports"
+               " WHERE ifnull(coalesce(euser,user,'')=%Q,1)"
+               " GROUP BY dow ORDER BY dow", zUserName);
+  @ <h1>Timeline Events (%h(stats_report_label_for_type())) by Day of the Week
+  if( zUserName ){
+    @ for user %h(zUserName)
+  }
+  @ </h1>
+  db_multi_exec(
+    "CREATE TEMP VIEW piechart(amt,label) AS"
+    " SELECT count(*),"
+    "   CASE cast(strftime('%%w', mtime) AS INT)"
+    "    WHEN 0 THEN 'Sunday'"
+    "    WHEN 1 THEN 'Monday'"
+    "    WHEN 2 THEN 'Tuesday'"
+    "    WHEN 3 THEN 'Wednesday'"
+    "    WHEN 4 THEN 'Thursday'"
+    "    WHEN 5 THEN 'Friday'"
+    "    WHEN 6 THEN 'Saturday'"
+    "    ELSE 'ERROR'"
+    "   END"
+    "  FROM v_reports"
+    "  WHERE ifnull(coalesce(euser,user,'')=%Q,1)"
+    "  GROUP BY 2 ORDER BY cast(strftime('%%w', mtime) AS INT);"
+    , zUserName
+  );
+  if( db_int(0, "SELECT count(*) FROM piechart")>=2 ){
+    @ <center><svg width=700 height=400>
+    piechart_render(700, 400, PIE_OTHER|PIE_PERCENT);
+    @ </svg></centre><hr/>
+  }
   @ <table class='statistics-report-table-events' border='0'
   @ cellpadding='2' cellspacing='0' id='statsTable'>
   @ <thead><tr>
@@ -574,136 +551,107 @@ static void stats_report_day_of_week(){
 /*
 ** Helper for stats_report_by_month_year(), which generates a list of
 ** week numbers. zTimeframe should be either a timeframe in the form YYYY
-** or YYYY-MM.
+** or YYYY-MM. If zUserName is not NULL then the report is restricted to events
+** created by the named user account.
 */
 static void stats_report_year_weeks(const char *zUserName){
-  const char *zYear = P("y");
-  int nYear = zYear ? strlen(zYear) : 0;
-  int i = 0;
-  Stmt qYears = empty_Stmt;
-  char *zDefaultYear = NULL;
-  Blob sql = empty_blob;
+  const char *zYear = P("y");        /* Year for which report shown */
+  Stmt q;
   int nMaxEvents = 1;                /* max number of events for
                                         all rows. */
   int iterations = 0;                /* # of active time periods. */
-  stats_report_init_view();
-  if(4==nYear){
-    Blob urlParams = empty_blob;
-    blob_appendf(&urlParams, "y=%T", zYear);
-    stats_report_event_types_menu("byweek", blob_str(&urlParams));
-    blob_reset(&urlParams);
-  }else{
-    stats_report_event_types_menu("byweek", NULL);
-  }
-  blob_append(&sql,
-              "SELECT DISTINCT substr(date(mtime),1,4) AS y "
-              "FROM v_reports WHERE 1 ", -1);
-  if(zUserName&&*zUserName){
-    blob_append_sql(&sql,"AND user=%Q ", zUserName);
-  }
-  blob_append(&sql,"GROUP BY y ORDER BY y", -1);
-  db_prepare(&qYears, "%s", blob_sql_text(&sql));
-  blob_reset(&sql);
-  cgi_printf("Select year: ");
-  while( SQLITE_ROW == db_step(&qYears) ){
-    const char *zT = db_column_text(&qYears, 0);
-    if( i++ ){
-      cgi_printf(" ");
-    }
-    cgi_printf("<a href='?view=byweek&y=%s&type=%c", zT,
-               (char)statsReportType);
-    if(zUserName && *zUserName){
-      cgi_printf("&user=%t",zUserName);
-    }
-    cgi_printf("'>%s</a>",zT);
-  }
-  db_finalize(&qYears);
-  cgi_printf("<br/>");
-  if(!zYear || !*zYear){
-    zDefaultYear = db_text("????", "SELECT strftime('%%Y')");
-    zYear = zDefaultYear;
-    nYear = 4;
-  }
-  if(4 == nYear){
-    Stmt stWeek = empty_Stmt;
-    int rowCount = 0;
-    int total = 0;
-    Blob header = empty_blob;
-    blob_appendf(&header, "Timeline events (%s) for the calendar weeks "
-                 "of %h", stats_report_label_for_type(),
-                 zYear);
-    blob_append_sql(&sql,
-                 "SELECT DISTINCT strftime('%%W',mtime) AS wk, "
-                 "count(*) AS n "
-                 "FROM v_reports "
-                 "WHERE %Q=substr(date(mtime),1,4) "
-                 "AND mtime < current_timestamp ",
-                 zYear);
-    if(zUserName&&*zUserName){
-      blob_append_sql(&sql, " AND user=%Q ", zUserName);
-      blob_appendf(&header," for user %h", zUserName);
-    }
-    blob_append_sql(&sql, "GROUP BY wk ORDER BY wk DESC");
-    cgi_printf("<h1>%h</h1>", blob_str(&header));
-    blob_reset(&header);
-    cgi_printf("<table class='statistics-report-table-events' "
-               "border='0' cellpadding='2' width='100%%' "
-               "cellspacing='0' id='statsTable'>");
-    cgi_printf("<thead><tr>"
-               "<th>Week</th>"
-               "<th>Events</th>"
-               "<th width='90%%'><!-- relative commits graph --></th>"
-               "</tr></thead>"
-               "<tbody>");
-    db_prepare(&stWeek, "%s", blob_sql_text(&sql));
-    blob_reset(&sql);
-    while( SQLITE_ROW == db_step(&stWeek) ){
-      const int nCount = db_column_int(&stWeek, 1);
-      if(nCount>nMaxEvents){
-        nMaxEvents = nCount;
-      }
-      ++iterations;
-    }
-    db_reset(&stWeek);
-    while( SQLITE_ROW == db_step(&stWeek) ){
-      const char *zWeek = db_column_text(&stWeek,0);
-      const int nCount = db_column_int(&stWeek,1);
-      int nSize = nCount
-        ? (int)(100 * nCount / nMaxEvents)
-        : 0;
-      if(!nSize) nSize = 1;
-      total += nCount;
-      cgi_printf("<tr class='row%d'>", ++rowCount % 2 );
-      cgi_printf("<td><a href='%R/timeline?yw=%t-%s&n=%d&y=%s",
-                 zYear, zWeek, nCount,
-                 statsReportTimelineYFlag);
-      if(zUserName && *zUserName){
-        cgi_printf("&u=%t",zUserName);
-      }
-      cgi_printf("'>%s</a></td>",zWeek);
+  int rowCount = 0;
+  int total = 0;
 
-      cgi_printf("<td>%d</td>",nCount);
-      cgi_printf("<td>");
-      if(nCount){
-        cgi_printf("<div class='statistics-report-graph-line'"
-                   "style='width:%d%%;'>&nbsp;</div>",
-                   nSize);
-      }
-      cgi_printf("</td></tr>");
-    }
-    db_finalize(&stWeek);
-    free(zDefaultYear);
-    cgi_printf("</tbody></table>");
-    if(total){
-      int nAvg = iterations ? (total/iterations) : 0;
-      cgi_printf("<br><div>Total events: %d<br>"
-                 "Average per active week: %d</div>",
-                 total, nAvg);
-    }
-    output_table_sorting_javascript("statsTable","tnx",-1);
+  stats_report_init_view();
+  style_submenu_sql("y", "Year:",
+     "WITH RECURSIVE a(b) AS ("
+     "  SELECT substr(date('now'),1,4) UNION ALL"
+     "  SELECT b-1 FROM a"
+     "   WHERE b>0+(SELECT substr(date(min(mtime)),1,4) FROM event)"
+     ") SELECT b, b FROM a ORDER BY b DESC");
+  if( zYear==0 || strlen(zYear)!=4 ){
+    zYear = db_text("1970","SELECT substr(date('now'),1,4);");
   }
+  cgi_printf("<br/>");
+  db_prepare(&q,
+             "SELECT DISTINCT strftime('%%W',mtime) AS wk, "
+             "       count(*) AS n "
+             "  FROM v_reports "
+             " WHERE %Q=substr(date(mtime),1,4) "
+             "   AND mtime < current_timestamp "
+             "   AND ifnull(coalesce(euser,user,'')=%Q,1)"
+             " GROUP BY wk ORDER BY wk DESC", zYear, zUserName);
+  @ <h1>Timeline events (%h(stats_report_label_for_type()))
+  @ for the calendar weeks of %h(zYear)
+  if( zUserName ){
+    @  for user %h(zUserName)
+  }
+  @ </h1>
+  cgi_printf("<table class='statistics-report-table-events' "
+              "border='0' cellpadding='2' width='100%%' "
+             "cellspacing='0' id='statsTable'>");
+  cgi_printf("<thead><tr>"
+             "<th>Week</th>"
+             "<th>Events</th>"
+             "<th width='90%%'><!-- relative commits graph --></th>"
+             "</tr></thead>"
+             "<tbody>");
+  while( SQLITE_ROW == db_step(&q) ){
+    const int nCount = db_column_int(&q, 1);
+    if(nCount>nMaxEvents){
+      nMaxEvents = nCount;
+    }
+    ++iterations;
+  }
+  db_reset(&q);
+  while( SQLITE_ROW == db_step(&q) ){
+    const char *zWeek = db_column_text(&q,0);
+    const int nCount = db_column_int(&q,1);
+    int nSize = nCount
+      ? (int)(100 * nCount / nMaxEvents)
+      : 0;
+    if(!nSize) nSize = 1;
+    total += nCount;
+    cgi_printf("<tr class='row%d'>", ++rowCount % 2 );
+    cgi_printf("<td><a href='%R/timeline?yw=%t-%s&n=%d&y=%s",
+               zYear, zWeek, nCount,
+               statsReportTimelineYFlag);
+    if( zUserName ){
+      cgi_printf("&u=%t",zUserName);
+    }
+    cgi_printf("'>%s</a></td>",zWeek);
+
+    cgi_printf("<td>%d</td>",nCount);
+    cgi_printf("<td>");
+    if(nCount){
+      cgi_printf("<div class='statistics-report-graph-line'"
+                 "style='width:%d%%;'>&nbsp;</div>",
+                 nSize);
+    }
+    cgi_printf("</td></tr>");
+  }
+  db_finalize(&q);
+  cgi_printf("</tbody></table>");
+  if(total){
+    int nAvg = iterations ? (total/iterations) : 0;
+    cgi_printf("<br><div>Total events: %d<br>"
+               "Average per active week: %d</div>",
+               total, nAvg);
+  }
+  output_table_sorting_javascript("statsTable","tnx",-1);
 }
 
+/* Report types
+*/
+#define RPT_BYFILE    1
+#define RPT_BYMONTH   2
+#define RPT_BYUSER    3
+#define RPT_BYWEEK    4
+#define RPT_BYWEEKDAY 5
+#define RPT_BYYEAR    6
+#define RPT_NONE      0  /* None of the above */
+
 /*
 ** WEBPAGE: reports
 **
@@ -725,50 +673,89 @@ static void stats_report_year_weeks(const char *zUserName){
 **                     current year).
 */
 void stats_report_page(){
-  HQuery url;                        /* URL for various branch links */
-  const char *zView = P("view");    /* Which view/report to show. */
-  const char *zUserName = P("user");
+  const char *zView = P("view");     /* Which view/report to show. */
+  int eType = RPT_NONE;              /* Numeric code for view/report to show */
+  int i;                             /* Loop counter */
+  const char *zUserName;             /* Name of user */
+  const char *azView[16];            /* Drop-down menu of view types */
+  static const struct {
+    const char *zName;  /* Name of view= screen type */
+    const char *zVal;   /* Value of view= query parameter */
+    int eType;          /* Corresponding RPT_* define */
+  } aViewType[] = {
+     {  "File Changes","byfile",    RPT_BYFILE    },
+     {  "By Month",    "bymonth",   RPT_BYMONTH   },
+     {  "By User",     "byuser",    RPT_BYUSER    },
+     {  "By Week",     "byweek",    RPT_BYWEEK    },
+     {  "By Weekday",  "byweekday", RPT_BYWEEKDAY },
+     {  "By Year",     "byyear",    RPT_BYYEAR   },
+  };
+  static const char *const azType[] = {
+     "a",  "All Changes",
+     "ci", "Check-ins",
+     "g",  "Tags",
+     "e",  "Tech Notes",
+     "t",  "Tickets",
+     "w",  "Wiki"
+  };
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  if(!zUserName) zUserName = P("u");
-  url_initialize(&url, "reports");
-  if(zUserName && *zUserName){
-    url_add_parameter(&url,"user", zUserName);
-    statrep_submenu(&url, "(Remove User Flag)", "view", zView, "user");
+  zUserName = P("user");
+  if( zUserName==0 ) zUserName = P("u");
+  if( zUserName && zUserName[0]==0 ) zUserName = 0;
+  if( zView==0 ){
+    zView = "byuser";
+    cgi_replace_query_parameter("view","byuser");
+  }
+  for(i=0; i<ArraySize(aViewType); i++){
+    if( fossil_strcmp(zView, aViewType[i].zVal)==0 ){
+      eType = aViewType[i].eType;
+      break;
+    }
+  }
+  if( eType!=RPT_NONE ){
+    int nView = 0;                     /* Slots used in azView[] */
+    for(i=0; i<ArraySize(aViewType); i++){
+      azView[nView++] = aViewType[i].zVal;
+      azView[nView++] = aViewType[i].zName;
+    }
+    if( eType!=RPT_BYFILE ){
+      style_submenu_multichoice("type", ArraySize(azType)/2, azType, 0);
+    }
+    style_submenu_multichoice("view", nView/2, azView, 0);
+    if( eType!=RPT_BYUSER ){
+      style_submenu_sql("user","User:",
+         "SELECT '', 'All Users' UNION ALL "
+         "SELECT x, x FROM ("
+         "  SELECT DISTINCT trim(coalesce(euser,user)) AS x FROM event %s"
+         "  ORDER BY 1 COLLATE nocase) WHERE x!=''",
+         eType==RPT_BYFILE ? "WHERE type='ci'" : ""
+      );
+    }
   }
-  statrep_submenu(&url, "By Year", "view", "byyear", 0);
-  statrep_submenu(&url, "By Month", "view", "bymonth", 0);
-  statrep_submenu(&url, "By Week", "view", "byweek", 0);
-  statrep_submenu(&url, "By Weekday", "view", "byweekday", 0);
-  statrep_submenu(&url, "By User", "view", "byuser", "user");
-  statrep_submenu(&url, "By File", "view", "byfile", "file");
   style_submenu_element("Stats", "Stats", "%R/stat");
-  url_reset(&url);
   style_header("Activity Reports");
-  if(0==fossil_strcmp(zView,"byyear")){
-    stats_report_by_month_year(0, 0, zUserName);
-  }else if(0==fossil_strcmp(zView,"bymonth")){
-    stats_report_by_month_year(1, 0, zUserName);
-  }else if(0==fossil_strcmp(zView,"byweek")){
-    stats_report_year_weeks(zUserName);
-  }else if(0==fossil_strcmp(zView,"byuser")){
-    stats_report_by_user();
-  }else if(0==fossil_strcmp(zView,"byweekday")){
-    stats_report_day_of_week();
-  }else if(0==fossil_strcmp(zView,"byfile")){
-    stats_report_by_file();
-  }else{
-    @ <h1>Activity Reports:</h1>
-    @ <ul>
-    @ <li>%z(href("?view=byyear"))Events by year</a></li>
-    @ <li>%z(href("?view=bymonth"))Events by month</a></li>
-    @ <li>%z(href("?view=byweek"))Events by calendar week</a></li>
-    @ <li>%z(href("?view=byweekday"))Events by day of the week</a></li>
-    @ <li>%z(href("?view=byuser"))Events by user</a></li>
-    @ <li>%z(href("?view=byfile"))Events by file</a></li>
-    @ </ul>
+  switch( eType ){
+    case RPT_BYYEAR:
+      stats_report_by_month_year(0, 0, zUserName);
+      break;
+    case RPT_BYMONTH:
+      stats_report_by_month_year(1, 0, zUserName);
+      break;
+    case RPT_BYWEEK:
+      stats_report_year_weeks(zUserName);
+      break;
+    default:
+    case RPT_BYUSER:
+      stats_report_by_user();
+      break;
+    case RPT_BYWEEKDAY:
+      stats_report_day_of_week(zUserName);
+      break;
+    case RPT_BYFILE:
+      stats_report_by_file(zUserName);
+      break;
   }
-
   style_footer();
 }
