@@ -145,6 +145,99 @@ void Th_PrintTraceLog(){
   }
 }
 
+/*
+** - adopted from ls_cmd_rev in checkin.c
+** - adopted commands/error handling for usage within th1
+** - interface adopted to allow result creation as TH1 List
+**
+** Takes a checkin identifier in zRev and an optiona glob pattern in zGLOB
+** as parameter returns a TH list in pzList,pnList with filenames matching
+** glob pattern with the checking
+*/
+static void dir_cmd_rev(
+  Th_Interp *interp,
+  char **pzList,
+  int *pnList,
+  const char *zRev,  /* Revision string given */
+  const char *zGlob, /* Glob pattern given */
+  int bDetails
+){
+  Stmt q;
+  char *zOrderBy = "pathname COLLATE nocase";
+  int rid;
+
+  rid = th1_name_to_typed_rid(interp, zRev, "ci");
+  compute_fileage(rid, zGlob);
+  db_prepare(&q,
+    "SELECT datetime(fileage.mtime, toLocal()), fileage.pathname,\n"
+    "       blob.size\n"
+    "  FROM fileage, blob\n"
+    " WHERE blob.rid=fileage.fid \n"
+    " ORDER BY %s;", zOrderBy /*safe-for-%s*/
+  );
+  while( db_step(&q)==SQLITE_ROW ){
+    const char *zFile = db_column_text(&q, 1);
+    if( bDetails ){
+      const char *zTime = db_column_text(&q, 0);
+      int size = db_column_int(&q, 2);
+      char zSize[50];
+      char *zSubList = 0;
+      int nSubList = 0;
+      sqlite3_snprintf(sizeof(zSize), zSize, "%d", size);
+      Th_ListAppend(interp, &zSubList, &nSubList, zFile, -1);
+      Th_ListAppend(interp, &zSubList, &nSubList, zSize, -1);
+      Th_ListAppend(interp, &zSubList, &nSubList, zTime, -1);
+      Th_ListAppend(interp, pzList, pnList, zSubList, -1);
+      Th_Free(interp, zSubList);
+    }else{
+      Th_ListAppend(interp, pzList, pnList, zFile, -1);
+    }
+  }
+  db_finalize(&q);
+}
+
+/*
+** TH1 command: dir CHECKIN ?GLOB? ?DETAILS?
+**
+** Returns a list containing all files in CHECKIN. If GLOB is given only
+** the files matching the pattern GLOB within CHECKIN will be returned.
+** If DETAILS is non-zero, the result will be a list-of-lists, with each
+** element containing at least three elements: the file name, the file
+** size (in bytes), and the file last modification time (relative to the
+** time zone configured for the repository).
+*/
+static int dirCmd(
+  Th_Interp *interp,
+  void *ctx,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  const char *zGlob = 0;
+  int bDetails = 0;
+
+  if( argc<2 || argc>4 ){
+    return Th_WrongNumArgs(interp, "dir CHECKIN ?GLOB? ?DETAILS?");
+  }
+  if( argc>=3 ){
+    zGlob = argv[2];
+  }
+  if( argc>=4 && Th_ToInt(interp, argv[3], argl[3], &bDetails) ){
+    return TH_ERROR;
+  }
+  if( Th_IsRepositoryOpen() ){
+    char *zList = 0;
+    int nList = 0;
+    dir_cmd_rev(interp, &zList, &nList, argv[1], zGlob, bDetails);
+    Th_SetResult(interp, zList, nList);
+    Th_Free(interp, zList);
+    return TH_OK;
+  }else{
+    Th_SetResult(interp, "repository unavailable", -1);
+    return TH_ERROR;
+  }
+}
+
 /*
 ** TH1 command: httpize STRING
 **
@@ -176,7 +269,7 @@ static int enableOutput = 1;
 /*
 ** TH1 command: enable_output BOOLEAN
 **
-** Enable or disable the puts and hputs commands.
+** Enable or disable the puts and wiki commands.
 */
 static int enableOutputCmd(
   Th_Interp *interp,
@@ -341,9 +434,105 @@ static int putsCmd(
 }
 
 /*
+** TH1 command: redirect URL
+**
+** Issues an HTTP redirect (302) to the specified URL and then exits the
+** process.
+*/
+static int redirectCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  if( argc!=2 ){
+    return Th_WrongNumArgs(interp, "redirect URL");
+  }
+  cgi_redirect(argv[1]);
+  Th_SetResult(interp, argv[1], argl[1]); /* NOT REACHED */
+  return TH_OK;
+}
+
+/*
+** TH1 command: insertCsrf
+**
+** While rendering a form, call this command to add the Anti-CSRF token
+** as a hidden element of the form.
+*/
+static int insertCsrfCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  if( argc!=1 ){
+    return Th_WrongNumArgs(interp, "insertCsrf");
+  }
+  login_insert_csrf_secret();
+  return TH_OK;
+}
+
+/*
+** TH1 command: verifyCsrf
+**
+** Before using the results of a form, first call this command to verify
+** that this Anti-CSRF token is present and is valid.  If the Anti-CSRF token
+** is missing or is incorrect, that indicates a cross-site scripting attack.
+** If the event of an attack is detected, an error message is generated and
+** all further processing is aborted.
+*/
+static int verifyCsrfCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  if( argc!=1 ){
+    return Th_WrongNumArgs(interp, "verifyCsrf");
+  }
+  login_verify_csrf_secret();
+  return TH_OK;
+}
+
+/*
+** TH1 command: markdown STRING
+**
+** Renders the input string as markdown.  The result is a two-element list.
+** The first element is the text-only title string.  The second element
+** contains the body, rendered as HTML.
+*/
+static int markdownCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  Blob src, title, body;
+  char *zValue = 0;
+  int nValue = 0;
+  if( argc!=2 ){
+    return Th_WrongNumArgs(interp, "markdown STRING");
+  }
+  blob_zero(&src);
+  blob_init(&src, (char*)argv[1], argl[1]);
+  blob_zero(&title); blob_zero(&body);
+  markdown_to_html(&src, &title, &body);
+  Th_ListAppend(interp, &zValue, &nValue, blob_str(&title), blob_size(&title));
+  Th_ListAppend(interp, &zValue, &nValue, blob_str(&body), blob_size(&body));
+  Th_SetResult(interp, zValue, nValue);
+  return TH_OK;
+}
+
+/*
+** TH1 command: decorate STRING
 ** TH1 command: wiki STRING
 **
-** Render the input string as wiki.
+** Render the input string as wiki.  For the decorate command, only links
+** are handled.
 */
 static int wikiCmd(
   Th_Interp *interp,
@@ -388,6 +577,28 @@ static int htmlizeCmd(
   return TH_OK;
 }
 
+/*
+** TH1 command: encode64 STRING
+**
+** Encode the specified string using Base64 and return the result.
+*/
+static int encode64Cmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  char *zOut;
+  if( argc!=2 ){
+    return Th_WrongNumArgs(interp, "encode64 STRING");
+  }
+  zOut = encode64((char*)argv[1], argl[1]);
+  Th_SetResult(interp, zOut, -1);
+  free(zOut);
+  return TH_OK;
+}
+
 /*
 ** TH1 command: date
 **
@@ -404,7 +615,7 @@ static int dateCmd(
 ){
   char *zOut;
   if( argc>=2 && argl[1]==6 && memcmp(argv[1],"-local",6)==0 ){
-    zOut = db_text("??", "SELECT datetime('now'%s)", timeline_utc());
+    zOut = db_text("??", "SELECT datetime('now',toLocal())");
   }else{
     zOut = db_text("??", "SELECT datetime('now')");
   }
@@ -507,6 +718,8 @@ static int searchableCmd(
 ** enabled. The set of features includes:
 **
 ** "ssl"             = FOSSIL_ENABLE_SSL
+** "legacyMvRm"      = FOSSIL_ENABLE_LEGACY_MV_RM
+** "execRelPaths"    = FOSSIL_ENABLE_EXEC_REL_PATHS
 ** "th1Docs"         = FOSSIL_ENABLE_TH1_DOCS
 ** "th1Hooks"        = FOSSIL_ENABLE_TH1_HOOKS
 ** "tcl"             = FOSSIL_ENABLE_TCL
@@ -516,7 +729,11 @@ static int searchableCmd(
 ** "json"            = FOSSIL_ENABLE_JSON
 ** "markdown"        = FOSSIL_ENABLE_MARKDOWN
 ** "unicodeCmdLine"  = !BROKEN_MINGW_CMDLINE
+** "dynamicBuild"    = FOSSIL_DYNAMIC_BUILD
+** "see"             = USE_SEE
 **
+** Specifying an unknown feature will return a value of false, it will not
+** raise a script error.
 */
 static int hasfeatureCmd(
   Th_Interp *interp,
@@ -539,6 +756,16 @@ static int hasfeatureCmd(
     rc = 1;
   }
 #endif
+#if defined(FOSSIL_ENABLE_LEGACY_MV_RM)
+  else if( 0 == fossil_strnicmp( zArg, "legacyMvRm\0", 11 ) ){
+    rc = 1;
+  }
+#endif
+#if defined(FOSSIL_ENABLE_EXEC_REL_PATHS)
+  else if( 0 == fossil_strnicmp( zArg, "execRelPaths\0", 13 ) ){
+    rc = 1;
+  }
+#endif
 #if defined(FOSSIL_ENABLE_TH1_DOCS)
   else if( 0 == fossil_strnicmp( zArg, "th1Docs\0", 8 ) ){
     rc = 1;
@@ -578,6 +805,16 @@ static int hasfeatureCmd(
   else if( 0 == fossil_strnicmp( zArg, "unicodeCmdLine\0", 15 ) ){
     rc = 1;
   }
+#endif
+#if defined(FOSSIL_DYNAMIC_BUILD)
+  else if( 0 == fossil_strnicmp( zArg, "dynamicBuild\0", 13 ) ){
+    rc = 1;
+  }
+#endif
+#if defined(USE_SEE)
+  else if( 0 == fossil_strnicmp( zArg, "see\0", 4 ) ){
+    rc = 1;
+  }
 #endif
   else if( 0 == fossil_strnicmp( zArg, "markdown\0", 9 ) ){
     rc = 1;
@@ -1319,6 +1556,51 @@ static int settingCmd(
   return rc;
 }
 
+/*
+** TH1 command: glob_match ?-one? ?--? patternList string
+**
+** Checks the string against the specified glob pattern -OR- list of glob
+** patterns and returns non-zero if there is a match.
+*/
+#define GLOB_MATCH_WRONGNUMARGS "glob_match ?-one? ?--? patternList string"
+static int globMatchCmd(
+  Th_Interp *interp,
+  void *p,
+  int argc,
+  const char **argv,
+  int *argl
+){
+  int rc;
+  int one = 0;
+  int nArg = 1;
+  Glob *pGlob = 0;
+  if( argc<3 || argc>5 ){
+    return Th_WrongNumArgs(interp, GLOB_MATCH_WRONGNUMARGS);
+  }
+  if( fossil_strcmp(argv[nArg], "-one")==0 ){
+    one = 1; nArg++;
+  }
+  if( fossil_strcmp(argv[nArg], "--")==0 ) nArg++;
+  if( nArg+2!=argc ){
+    return Th_WrongNumArgs(interp, GLOB_MATCH_WRONGNUMARGS);
+  }
+  if( one ){
+    Th_SetResultInt(interp, sqlite3_strglob(argv[nArg], argv[nArg+1])==0);
+    rc = TH_OK;
+  }else{
+    pGlob = glob_create(argv[nArg]);
+    if( pGlob ){
+      Th_SetResultInt(interp, glob_match(pGlob, argv[nArg+1]));
+      rc = TH_OK;
+    }else{
+      Th_SetResult(interp, "unable to create glob from pattern list", -1);
+      rc = TH_ERROR;
+    }
+    glob_free(pGlob);
+  }
+  return rc;
+}
+
 /*
 ** TH1 command: regexp ?-nocase? ?--? exp string
 **
@@ -1492,7 +1774,7 @@ void Th_OpenConfig(
     }
   }
   if( !Th_IsConfigOpen() ){
-    db_open_config(0);
+    db_open_config(0, 1);
     if( Th_IsConfigOpen() ){
       g.th1Flags |= TH_STATE_CONFIG;
     }else{
@@ -1545,8 +1827,11 @@ void Th_FossilInit(u32 flags){
     {"combobox",      comboboxCmd,          0},
     {"date",          dateCmd,              0},
     {"decorate",      wikiCmd,              (void*)&aFlags[2]},
+    {"dir",           dirCmd,               0},
     {"enable_output", enableOutputCmd,      0},
+    {"encode64",      encode64Cmd,          0},
     {"getParameter",  getParameterCmd,      0},
+    {"glob_match",    globMatchCmd,         0},
     {"globalState",   globalStateCmd,       0},
     {"httpize",       httpizeCmd,           0},
     {"hascap",        hascapCmd,            (void*)&zeroInt},
@@ -1554,10 +1839,13 @@ void Th_FossilInit(u32 flags){
     {"html",          putsCmd,              (void*)&aFlags[0]},
     {"htmlize",       htmlizeCmd,           0},
     {"http",          httpCmd,              0},
+    {"insertCsrf",    insertCsrfCmd,        0},
     {"linecount",     linecntCmd,           0},
+    {"markdown",      markdownCmd,          0},
     {"puts",          putsCmd,              (void*)&aFlags[1]},
     {"query",         queryCmd,             0},
     {"randhex",       randhexCmd,           0},
+    {"redirect",      redirectCmd,          0},
     {"regexp",        regexpCmd,            0},
     {"reinitialize",  reinitializeCmd,      0},
     {"render",        renderCmd,            0},
@@ -1571,6 +1859,7 @@ void Th_FossilInit(u32 flags){
     {"trace",         traceCmd,             0},
     {"stime",         stimeCmd,             0},
     {"utime",         utimeCmd,             0},
+    {"verifyCsrf",    verifyCsrfCmd,        0},
     {"wiki",          wikiCmd,              (void*)&aFlags[0]},
     {0, 0, 0}
   };
@@ -1835,6 +2124,12 @@ int Th_CommandHook(
     */
     if( memcmp(zResult, NO_COMMAND_HOOK_ERROR, nResult)!=0 ){
       sendError(zResult, nResult, 0);
+    }else{
+      /*
+      ** There is no command hook handler "installed".  This situation
+      ** is NOT actually an error.
+      */
+      rc = TH_OK;
     }
   }
   /*
@@ -1855,7 +2150,7 @@ int Th_CommandHook(
   ** open prior to their own code doing so.
   */
   if( TH_INIT_HOOK & TH_INIT_NEED_CONFIG ) Th_CloseConfig(1);
-  return (rc != TH_ERROR) ? rc : TH_OK;
+  return rc;
 }
 
 /*
@@ -1916,6 +2211,12 @@ int Th_WebpageHook(
     */
     if( memcmp(zResult, NO_WEBPAGE_HOOK_ERROR, nResult)!=0 ){
       sendError(zResult, nResult, 1);
+    }else{
+      /*
+      ** There is no webpage hook handler "installed".  This situation
+      ** is NOT actually an error.
+      */
+      rc = TH_OK;
     }
   }
   /*
@@ -1936,7 +2237,7 @@ int Th_WebpageHook(
   ** open prior to their own code doing so.
   */
   if( TH_INIT_HOOK & TH_INIT_NEED_CONFIG ) Th_CloseConfig(1);
-  return (rc != TH_ERROR) ? rc : TH_OK;
+  return rc;
 }
 
 /*
@@ -1972,6 +2273,20 @@ int Th_WebpageNotify(
 }
 #endif
 
+
+#ifdef FOSSIL_ENABLE_TH1_DOCS
+/*
+** This function determines if TH1 docs are enabled for the repository.
+*/
+int Th_AreDocsEnabled(void){
+  if( fossil_getenv("TH1_ENABLE_DOCS")!=0 ){
+    return 1;
+  }
+  return db_get_boolean("th1-docs", 0);
+}
+#endif
+
+
 /*
 ** The z[] input contains text mixed with TH1 scripts.
 ** The TH1 scripts are contained within <th1>...</th1>.
@@ -2048,9 +2363,10 @@ int Th_Render(const char *z){
 **     --cgi                Include a CGI response header in the output
 **     --http               Include an HTTP response header in the output
 **     --open-config        Open the configuration database
+**     --th-trace           Trace TH1 execution (for debugging purposes)
 */
 void test_th_render(void){
-  int forceCgi = 0, fullHttpReply = 0;
+  int forceCgi, fullHttpReply;
   Blob in;
   Th_InitTraceLog();
   forceCgi = find_option("cgi", 0, 0)!=0;
@@ -2077,13 +2393,14 @@ void test_th_render(void){
 ** Usage: %fossil test-th-eval SCRIPT
 **
 ** Evaluate SCRIPT as if it were a header or footer or ticket rendering
-** script, evaluate it, and show the results on standard output.
+** script and show the results on standard output.
 **
 ** Options:
 **
 **     --cgi                Include a CGI response header in the output
 **     --http               Include an HTTP response header in the output
 **     --open-config        Open the configuration database
+**     --th-trace           Trace TH1 execution (for debugging purposes)
 */
 void test_th_eval(void){
   int rc;
@@ -2097,6 +2414,7 @@ void test_th_eval(void){
   if( find_option("open-config", 0, 0)!=0 ){
     Th_OpenConfig(1);
   }
+  verify_all_options();
   if( g.argc!=3 ){
     usage("script");
   }
@@ -2108,20 +2426,97 @@ void test_th_eval(void){
   if( forceCgi ) cgi_reply();
 }
 
+/*
+** COMMAND: test-th-source
+**
+** Usage: %fossil test-th-source FILE
+**
+** Evaluate the contents of the file named "FILE" as if it were a header
+** or footer or ticket rendering script and show the results on standard
+** output.
+**
+** Options:
+**
+**     --cgi                Include a CGI response header in the output
+**     --http               Include an HTTP response header in the output
+**     --open-config        Open the configuration database
+**     --th-trace           Trace TH1 execution (for debugging purposes)
+*/
+void test_th_source(void){
+  int rc;
+  const char *zRc;
+  int forceCgi, fullHttpReply;
+  Blob in;
+  Th_InitTraceLog();
+  forceCgi = find_option("cgi", 0, 0)!=0;
+  fullHttpReply = find_option("http", 0, 0)!=0;
+  if( fullHttpReply ) forceCgi = 1;
+  if( forceCgi ) Th_ForceCgi(fullHttpReply);
+  if( find_option("open-config", 0, 0)!=0 ){
+    Th_OpenConfig(1);
+  }
+  verify_all_options();
+  if( g.argc!=3 ){
+    usage("file");
+  }
+  blob_zero(&in);
+  blob_read_from_file(&in, g.argv[2]);
+  Th_FossilInit(TH_INIT_DEFAULT);
+  rc = Th_Eval(g.interp, 0, blob_str(&in), -1);
+  zRc = Th_ReturnCodeName(rc, 1);
+  fossil_print("%s%s%s\n", zRc, zRc ? ": " : "", Th_GetResult(g.interp, 0));
+  Th_PrintTraceLog();
+  if( forceCgi ) cgi_reply();
+}
+
 #ifdef FOSSIL_ENABLE_TH1_HOOKS
 /*
 ** COMMAND: test-th-hook
+**
+** Usage: %fossil test-th-hook TYPE NAME FLAGS
+**
+** Evaluates the TH1 script configured for the pre-operation (i.e. a command
+** or web page) "hook" or post-operation "notification".  The results of the
+** script evaluation, if any, will be printed to the standard output channel.
+** The NAME argument must be the name of a command or web page; however, it
+** does not necessarily have to be a command or web page that is normally
+** recognized by Fossil.  The FLAGS argument will be used to set the value
+** of the "cmd_flags" and/or "web_flags" TH1 variables, if applicable.  The
+** TYPE argument must be one of the following:
+**
+**     cmdhook              Executes the TH1 procedure [command_hook], after
+**                          setting the TH1 variables "cmd_name", "cmd_args",
+**                          and "cmd_flags" to appropriate values.
+**
+**     cmdnotify            Executes the TH1 procedure [command_notify], after
+**                          setting the TH1 variables "cmd_name", "cmd_args",
+**                          and "cmd_flags" to appropriate values.
+**
+**     webhook              Executes the TH1 procedure [webpage_hook], after
+**                          setting the TH1 variables "web_name", "web_args",
+**                          and "web_flags" to appropriate values.
+**
+**     webnotify            Executes the TH1 procedure [webpage_notify], after
+**                          setting the TH1 variables "web_name", "web_args",
+**                          and "web_flags" to appropriate values.
+**
+** Options:
+**
+**     --cgi                Include a CGI response header in the output
+**     --http               Include an HTTP response header in the output
+**     --th-trace           Trace TH1 execution (for debugging purposes)
 */
 void test_th_hook(void){
   int rc = TH_OK;
   int nResult = 0;
-  char *zResult;
+  char *zResult = 0;
   int forceCgi, fullHttpReply;
   Th_InitTraceLog();
   forceCgi = find_option("cgi", 0, 0)!=0;
   fullHttpReply = find_option("http", 0, 0)!=0;
   if( fullHttpReply ) forceCgi = 1;
   if( forceCgi ) Th_ForceCgi(fullHttpReply);
+  verify_all_options();
   if( g.argc<5 ){
     usage("TYPE NAME FLAGS");
   }
@@ -2136,11 +2531,16 @@ void test_th_hook(void){
   }else{
     fossil_fatal("Unknown TH1 hook %s\n", g.argv[2]);
   }
-  zResult = (char*)Th_GetResult(g.interp, &nResult);
+  if( g.interp ){
+    zResult = (char*)Th_GetResult(g.interp, &nResult);
+  }
   sendText("RESULT (", -1, 0);
   sendText(Th_ReturnCodeName(rc, 0), -1, 0);
-  sendText("): ", -1, 0);
-  sendText(zResult, nResult, 0);
+  sendText(")", -1, 0);
+  if( zResult && nResult>0 ){
+    sendText(": ", -1, 0);
+    sendText(zResult, nResult, 0);
+  }
   sendText("\n", -1, 0);
   Th_PrintTraceLog();
   if( forceCgi ) cgi_reply();
