@@ -469,7 +469,7 @@ static int wikiUsesHtml(void){
 ** the markup including the initial "<" and the terminating ">".  If
 ** it is not well-formed markup, return 0.
 */
-static int markupLength(const char *z){
+int htmlTagLength(const char *z){
   int n = 1;
   int inparen = 0;
   int c;
@@ -664,7 +664,7 @@ static int linkLength(const char *z){
 static int nextWikiToken(const char *z, Renderer *p, int *pTokenType){
   int n;
   if( z[0]=='<' ){
-    n = markupLength(z);
+    n = htmlTagLength(z);
     if( n>0 ){
       *pTokenType = TOKEN_MARKUP;
       return n;
@@ -1153,7 +1153,9 @@ static const char *validWikiPageName(Renderer *p, const char *zTarget){
   if( strcmp(zTarget, "Sandbox")==0 ) return zTarget;
   if( wiki_name_is_wellformed((const unsigned char *)zTarget)
    && ((p->state & WIKI_NOBADLINKS)==0 ||
-        db_exists("SELECT 1 FROM tag WHERE tagname GLOB 'wiki-%q'", zTarget))
+        db_exists("SELECT 1 FROM tag WHERE tagname GLOB 'wiki-%q'"
+                  " AND (SELECT value FROM tagxref WHERE tagid=tag.tagid"
+                  " ORDER BY mtime DESC LIMIT 1) > 0", zTarget))
   ){
     return zTarget;
   }
@@ -1252,6 +1254,8 @@ static void openHyperlink(
     }else if( g.perm.Hyperlink ){
       blob_appendf(p->pOut, "%z[",href("%R/info/%s", zTarget));
       zTerm = "]</a>";
+    }else{
+      zTerm = "";
     }
   }else if( strlen(zTarget)>=10 && fossil_isdigit(zTarget[0]) && zTarget[4]=='-'
             && db_int(0, "SELECT datetime(%Q) NOT NULL", zTarget) ){
@@ -1717,7 +1721,7 @@ void wiki_write(const char *zIn, int flags){
 /*
 ** COMMAND: test-wiki-render
 **
-** %fossil test-wiki-render FILE [OPTIONS]
+** Usage: %fossil test-wiki-render FILE [OPTIONS]
 **
 ** Options:
 **    --buttons        Set the WIKI_BUTTONS flag
@@ -1970,7 +1974,7 @@ static int nextHtmlToken(const char *z){
   int n;
   char c;
   if( (c=z[0])=='<' ){
-    n = markupLength(z);
+    n = htmlTagLength(z);
     if( n<=0 ) n = 1;
   }else if( fossil_isspace(c) ){
     for(n=1; z[n] && fossil_isspace(z[n]); n++){}
@@ -2094,6 +2098,9 @@ void htmlTidy(const char *zIn, Blob *pOut){
 
 /*
 ** COMMAND: test-html-tidy
+**
+** Run the htmlTidy() routine on the content of all files named on
+** the command-line and write the results to standard output.
 */
 void test_html_tidy(void){
   Blob in, out;
@@ -2212,6 +2219,15 @@ void html_to_plaintext(const char *zIn, Blob *pOut){
 
 /*
 ** COMMAND: test-html-to-text
+**
+** Usage: %fossil test-html-to-text FILE ...
+**
+** Read all files named on the command-line.  Convert the file
+** content from HTML to text and write the results on standard
+** output.
+**
+** This command is intended as a test and debug interface for
+** the html_to_plaintext() routine.
 */
 void test_html_to_text(void){
   Blob in, out;
